@@ -134,9 +134,15 @@ printf "<input type=radio name=unix value=0 %s> $text{'no'}</td> </tr>\n",
 print "<tr> <td><b>$text{'acl_sessions'}</b></td> <td>\n";
 printf "<input type=radio name=sessions value=1 %s> $text{'yes'}\n",
 	$o->{'sessions'} ? 'checked' : '';
-printf "<input type=radio name=sessions value=0 %s> $text{'no'}</td> </tr>\n",
+printf "<input type=radio name=sessions value=0 %s> $text{'no'}</td>\n",
 	$o->{'sessions'} ? '' : 'checked';
 
+print "<td><b>$text{'acl_switch'}</b></td> <td>\n";
+printf "<input type=radio name=switch value=1 %s> $text{'yes'}\n",
+	$o->{'switch'} ? 'checked' : '';
+printf "<input type=radio name=switch value=0 %s> $text{'no'}</td> </tr>\n",
+	$o->{'switch'} ? '' : 'checked';
+
 print "<tr> <td colspan=4><hr></td> </tr>\n";
 
 print "<tr> <td valign=top><b>$text{'acl_groups'}</b></td> <td valign=top>\n";
@@ -191,6 +197,7 @@ $_[0]->{'gassign'} = $in{'gassign_def'} ? '*' :
 		     join(" ", split(/\0/, $in{'gassign'}));
 $_[0]->{'sync'} = $in{'sync'};
 $_[0]->{'unix'} = $in{'unix'};
+$_[0]->{'switch'} = $in{'switch'};
 $_[0]->{'sessions'} = $in{'sessions'};
 $_[0]->{'cats'} = $in{'cats'};
 $_[0]->{'theme'} = $in{'theme'};
