@@ -1,5 +1,5 @@
 
-do './acl-lib.pl';
+do 'acl-lib.pl';
 
 sub feedback_files
 {
