@@ -43,6 +43,9 @@ elsif ($action eq 'reset') {
 elsif ($action eq 'cert') {
 	return &text('log_cert', "<tt>".&html_escape($object)."</tt>");
 	}
+elsif ($action eq 'switch') {
+	return &text('log_switch', "<tt>".&html_escape($object)."</tt>");
+	}
 else {
 	return undef;
 	}
