@@ -14,7 +14,7 @@ local ($clash) = grep { $_->{'name'} eq $_[0]->{'user'} }
 return if ($clash);
 return if ($_[0]->{'user'} !~ /^[A-z0-9\-\_\.]+$/);
 local $user = { 'name' => $_[0]->{'user'},
-		'pass' => $_[0]->{'pass'},
+		'pass' => $config{'sync_unix'} ? 'x' : $_[0]->{'pass'},
 		'sync' => 1,
 		'modules' => $group->{'modules'} };
 &create_user($user);
@@ -71,9 +71,11 @@ foreach $u (&list_users()) {
 			return if ($_[0]->{'user'} !~ /^[A-z0-9\-\_\.]+$/);
 			}
 		$u->{'name'} = $_[0]->{'user'};
-		$u->{'pass'} = $_[0]->{'passmode'} == 3 ?
-			crypt($_[0]->{'plainpass'}, substr(time(), 0, 2)) :
-			$_[0]->{'pass'};
+		if ($u->{'pass'} ne 'x') {
+			$u->{'pass'} = $_[0]->{'passmode'} == 3 ?
+			   &encrypt_password($_[0]->{'plainpass'}) :
+			   $_[0]->{'pass'};
+			}
 		&modify_user($_[0]->{'olduser'}, $u);
 		&restart_miniserv();
 		}
