@@ -3,8 +3,9 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 
-do "./$config{'at_style'}-lib.pl";
+do "$config{'at_style'}-lib.pl";
 
 # wrap_lines(text, width)
 # Given a multi-line string, return an array of lines wrapped to
