@@ -32,10 +32,16 @@ sub create_atjob
 local @tm = localtime($_[1]);
 local $date = sprintf "%2.2d:%2.2d %d.%d.%d",
 		$tm[2], $tm[1], $tm[3], $tm[4]+1, $tm[5]+1900;
-open(AT, "| su \"$_[0]\" -c \"cd $_[3] ; at $date\" >/dev/null 2>&1"); 
+local $cmd = "cd $_[3]; at $date";
+local @uinfo = getpwnam($_[0]);
+if ($uinfo[2] != $<) {
+	# Only SU if we are not already the user
+	$cmd = "su \"$_[0]\" -c \"$cmd\"";
+	}
+open(AT, "| $cmd >/dev/null 2>&1"); 
 print AT $_[2];
 close(AT);
-&additional_log('exec', undef, "su \"$_[0]\" -c \"cd $_[3] ; at $date\"");
+&additional_log('exec', undef, $cmd);
 }
 
 # delete_atjob(id)
