@@ -29,11 +29,13 @@ while(<USERS>) {
 	local @user = split(/:/, $_);
 	if (@user) {
 		$users{$user[0]} = \@user;
+		push(@users, $user[0]);
 		}
 	}
 close(USERS);
 if (!defined($users{$user})) {
 	print STDERR "The Webmin user $user does not exist\n";
+	print STDERR "The users on your system are: ",join(" ", @users),"\n";
 	exit 5;
 	}
 $salt = substr(time(), 0, 2);
