@@ -2,18 +2,25 @@
 # Common crontab functions
 # XXX support for envs in /etc/crontab and /etc/cron.d (impossible!)
 
-
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 %access = &get_module_acl();
-$cron_temp_file = &tempname();
 $env_support = $config{'vixie_cron'};
+if ($module_info{'usermin'}) {
+	$single_user = $remote_user;
+	&switch_to_remote_user();
+	}
+$cron_temp_file = &tempname();
 
 # list_cron_jobs()
 # Returns a lists of structures of all cron jobs
 sub list_cron_jobs
 {
 local (@rv, $lnum, $f);
+if (defined(@cron_jobs_cache)) {
+	return @cron_jobs_cache;
+	}
 
 # read the master crontab file
 if ($config{'system_crontab'}) {
@@ -94,12 +101,22 @@ closedir(DIR);
 
 # read per-user cron files
 local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
-opendir(DIR, $config{'cron_dir'});
-while($f = readdir(DIR)) {
-	next if ($f =~ /^\./ || !(@uinfo = getpwnam($f)));
+local @users;
+if ($single_user) {
+	@users = ( $single_user );
+	}
+else {
+	opendir(DIR, $config{'cron_dir'});
+	@users = grep { !/^\./ } readdir(DIR);
+	closedir(DIR);
+	}
+foreach $f (@users) {
+	next if (!(@uinfo = getpwnam($f)));
 	$lnum = 0;
-	open(TAB, $fcron ? &user_sub($config{'cron_get_command'}, $f)." |"
-			 : "$config{'cron_dir'}/$f");
+	open(TAB, $single_user ? "$config{'cron_user_get_command'} |" :
+		  $fcron ?
+		  &user_sub($config{'cron_get_command'}, $f)." |" :
+		  "$config{'cron_dir'}/$f");
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
 			# A normal m h d m wd time
@@ -112,6 +129,7 @@ while($f = readdir(DIR)) {
 				    'weekdays' => $8, 'user' => $f,
 				    'command' => $10,
 				    'index' => scalar(@rv) });
+			$rv[$#rv]->{'file'} =~ s/\s+\|$//;
 			&fix_names($rv[$#rv]);
 			}
 		elsif (/^(#+)?\s*@([a-z]+)\s+(.*)/i) {
@@ -142,7 +160,8 @@ while($f = readdir(DIR)) {
 	close(TAB);
 	}
 closedir(DIR);
-return @rv;
+@cron_jobs_cache = @rv;
+return @cron_jobs_cache;
 }
 
 sub cron_job_line
@@ -172,38 +191,88 @@ else {
 return join(" ", @c);
 }
 
+# copy_cron_temp(&job)
+sub copy_cron_temp
+{
+local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
+unlink($cron_temp_file);
+if ($single_user) {
+	system("$config{'cron_user_get_command'} >$cron_temp_file 2>/dev/null");
+	}
+elsif ($fcron) {
+	system(&user_sub($config{'cron_get_command'},$_[0]->{'user'}).
+	       " >$cron_temp_file 2>/dev/null");
+	}
+else {
+	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file 2>/dev/null");
+	}
+}
+
 # create_cron_job(&job)
 # Add a Cron job to a user's file
 sub create_cron_job
 {
-unlink($cron_temp_file);
-system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file 2>/dev/null");
+&list_cron_jobs();	# init cache
+&copy_cron_temp($_[0]);
 local $lref = &read_file_lines($cron_temp_file);
+$_[0]->{'line'} = scalar(@$lref);
 push(@$lref, &cron_job_line($_[0]));
 &flush_file_lines();
 system("chown $_[0]->{'user'} $cron_temp_file");
 &copy_crontab($_[0]->{'user'});
+$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
+$_[0]->{'index'} = scalar(@cron_jobs_cache);
+push(@cron_jobs_cache, $_[0]);
 }
 
 # insert_cron_job(&job)
 # Add a Cron job at the top of the user's file
 sub insert_cron_job
 {
-unlink($cron_temp_file);
-system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file 2>/dev/null");
+&list_cron_jobs();	# init cache
+&copy_cron_temp($_[0]);
 local $lref = &read_file_lines($cron_temp_file);
+$_[0]->{'line'} = 0;
 splice(@$lref, 0, 0, &cron_job_line($_[0]));
 &flush_file_lines();
 system("chown $_[0]->{'user'} $cron_temp_file");
 &copy_crontab($_[0]->{'user'});
+$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
+$_[0]->{'index'} = scalar(@cron_jobs_cache);
+&renumber($_[0]->{'file'}, $_[0]->{'line'}, 1);
+push(@cron_jobs_cache, $_[0]);
+}
+
+# renumber(file, line, offset)
+# All jobs in this file whose line is at or after the given one will be
+# incremented by the offset
+sub renumber
+{
+local $j;
+foreach $j (@cron_jobs_cache) {
+	if ($j->{'line'} >= $_[1] &&
+	    $j->{'file'} eq $_[0]) {
+		$j->{'line'} += $_[2];
+		}
+	}
+}
+
+# renumber_index(index, offset)
+sub renumber_index
+{
+local $j;
+foreach $j (@cron_jobs_cache) {
+	if ($j->{'index'} >= $_[0]) {
+		$j->{'index'} += $_[1];
+		}
+	}
 }
 
 # change_cron_job(&job)
 sub change_cron_job
 {
 if ($_[0]->{'type'} == 0) {
-	unlink($cron_temp_file);
-	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file");
+	&copy_cron_temp($_[0]);
 	&replace_file_line($cron_temp_file, $_[0]->{'line'},
 			   &cron_job_line($_[0])."\n");
 	&copy_crontab($_[0]->{'user'});
@@ -218,14 +287,16 @@ else {
 sub delete_cron_job
 {
 if ($_[0]->{'type'} == 0) {
-	unlink($cron_temp_file);
-	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file");
+	&copy_cron_temp($_[0]);
 	&replace_file_line($cron_temp_file, $_[0]->{'line'});
 	&copy_crontab($_[0]->{'user'});
 	}
 else {
 	&replace_file_line($_[0]->{'file'}, $_[0]->{'line'});
 	}
+@cron_jobs_cache = grep { $_ ne $_[0] } @cron_jobs_cache;
+&renumber($_[0]->{'file'}, $_[0]->{'line'}, -1);
+&renumber_index($_[0]->{'index'}, -1);
 }
 
 # read_crontab(user)
@@ -259,17 +330,32 @@ if (`cat $cron_temp_file` =~ /\S/) {
 		print NO "N\n";
 		print NO "no\n";
 		close(NO);
-		$ENV{"VISUAL"} = $ENV{"EDITOR"} = "$module_root_directory/cron_editor.pl";
+		$ENV{"VISUAL"} = $ENV{"EDITOR"} =
+			"$module_root_directory/cron_editor.pl";
 		$ENV{"CRON_EDITOR_COPY"} = $cron_temp_file;
 		system("chown $_[0] $cron_temp_file");
-		$rv = system(&user_sub($config{'cron_edit_command'},$_[0]).
-		       " >$temp 2>&1 <$notemp");
+		if ($single_user) {
+			$rv = system($config{'cron_user_edit_command'}.
+				     " >$temp 2>&1 <$notemp");
+			}
+		else {
+			$rv = system(
+				&user_sub($config{'cron_edit_command'},$_[0]).
+				" >$temp 2>&1 <$notemp");
+			}
 		unlink($notemp);
 		}
 	else {
 		# use the cron copy command
-		$rv = system(&user_sub($config{'cron_copy_command'},$_[0]).
-		       " <$cron_temp_file >$temp 2>&1");
+		if ($single_user) {
+			$rv = system($config{'cron_user_copy_command'}.
+				     " <$cron_temp_file >$temp 2>&1");
+			}
+		else {
+			$rv = system(
+				&user_sub($config{'cron_copy_command'},$_[0]).
+				" <$cron_temp_file >$temp 2>&1");
+			}
 		}
 	local $out = `cat $temp`;
 	unlink($temp);
@@ -279,8 +365,15 @@ if (`cat $cron_temp_file` =~ /\S/) {
 		}
 	}
 else {
-	system(&user_sub($config{cron_delete_command},$_[0]).
-	       " >/dev/null 2>/dev/null");
+	# No more cron jobs left, so just delete
+	if ($single_user) {
+		system($config{'cron_user_delete_command'}.
+		       " >/dev/null 2>/dev/null");
+		}
+	else {
+		system(&user_sub($config{'cron_delete_command'},$_[0]).
+		       " >/dev/null 2>/dev/null");
+		}
 	}
 unlink($cron_temp_file);
 }
@@ -636,7 +729,7 @@ close(CONF);
 \$ENV{'WEBMIN_CONFIG'} = "$ENV{'WEBMIN_CONFIG'}";
 \$ENV{'WEBMIN_VAR'} = "$ENV{'WEBMIN_VAR'}";
 chdir("\$root/$_[1]");
-exec("\$root/$_[1]/$_[2]", \@ARGV);
+exec("\$root/$_[1]/$_[2]", \@ARGV) || die "Failed to run \$root/$_[1]/$_[2] : \$!";
 EOF
 close(CMD);
 chmod(0755, $_[0]);
@@ -649,30 +742,32 @@ sub cron_file
 return $_[0]->{'file'} || "$config{'cron_dir'}/$_[0]->{'user'}";
 }
 
-# when_text(&job)
+# when_text(&job, [upper-case-first])
 # Returns a text string describing when a cron job is run
 sub when_text
 {
+local $pfx = $_[1] ? "uc" : "";
 if ($_[0]->{'special'}) {
-	return $text{'edit_special_'.$_[0]->{'special'}};
+	$pfx = $_[1] ? "" : "lc";
+	return $text{$pfx.'edit_special_'.$_[0]->{'special'}};
 	}
 elsif ($_[0]->{'mins'} eq '*' && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return $text{'when_min'};
+	return $text{$pfx.'when_min'};
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text('when_hour', $_[0]->{'mins'});
+	return &text($pfx.'when_hour', $_[0]->{'mins'});
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text('when_day', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'});
+	return &text($pfx.'when_day', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'});
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} =~ /^\d+$/ && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text('when_month', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $_[0]->{'days'});
+	return &text($pfx.'when_month', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $_[0]->{'days'});
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} =~ /^\d+$/) {
-	return &text('when_weekday', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $text{"day_".$_[0]->{'weekdays'}});
+	return &text($pfx.'when_weekday', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $text{"day_".$_[0]->{'weekdays'}});
 	}
 else {
-	return &text('when_cron', join(" ", $_[0]->{'mins'}, $_[0]->{'hours'}, $_[0]->{'days'}, $_[0]->{'months'}, $_[0]->{'weekdays'}));
+	return &text($pfx.'when_cron', join(" ", $_[0]->{'mins'}, $_[0]->{'hours'}, $_[0]->{'days'}, $_[0]->{'months'}, $_[0]->{'weekdays'}));
 	}
 }
 
@@ -704,8 +799,7 @@ return !$err;
 sub swap_cron_jobs
 {
 if ($_[0]->{'type'} == 0) {
-	unlink($cron_temp_file);
-	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file");
+	&copy_cron_temp($_[0]);
 	local $lref = &read_file_lines($cron_temp_file);
 	($lref->[$_[0]->{'line'}], $lref->[$_[1]->{'line'}]) =
 		($lref->[$_[1]->{'line'}], $lref->[$_[0]->{'line'}]);
