@@ -1,5 +1,5 @@
 
-do './cron-lib.pl';
+do 'cron-lib.pl';
 
 # useradmin_create_user(&details)
 sub useradmin_create_user
