@@ -1,11 +1,11 @@
 
-require './fdisk-lib.pl';
+require 'fdisk-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the fdisk module
 sub acl_security_form
 {
-local @dlist = &list_disks();
+local @dlist = &list_disks_partitions();
 local ($d, %dcan);
 map { $dcan{$_}++ } split(/\s+/, $_[0]->{'disks'});
 print "<tr> <td valign=top><b>$text{'acl_disks'}</b></td> <td>\n";
