@@ -3,6 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 &foreign_require("mount", "mount-lib.pl");
 if (&foreign_check("raid")) {
 	&foreign_require("raid", "raid-lib.pl");
@@ -26,20 +27,78 @@ if (defined(@list_disks_partitions_cache)) {
 	return @list_disks_partitions_cache;
 	}
 
-# Check /proc/scsi for SCSI disk models
-local @pscsi;
-open(SCSI, "/proc/scsi/scsi");
-while(<SCSI>) {
-	s/\s/ /g;
-	if (/^Host:/) {
-		push(@pscsi, $_);
+local (@pscsi, @dscsi, $dscsi_mode);
+if (-r "/proc/scsi/sg/devices" && -r "/proc/scsi/sg/device_strs") {
+	# Get device info from various /proc/scsi files
+	open(DEVICES, "/proc/scsi/sg/devices");
+	while(<DEVICES>) {
+		s/\r|\n//g;
+		local @l = split(/\t+/, $_);
+		push(@dscsi, { 'host' => $l[0],
+			       'bus' => $l[1],
+			       'target' => $l[2],
+			       'lun' => $l[3],
+			       'type' => $l[4] });
 		}
-	elsif (/^\s+\S/) {
-		$pscsi[$#pscsi] .= $_;
+	close(DEVICES);
+	local $i = 0;
+	open(DEVNAMES, "/proc/scsi/sg/device_strs");
+	while(<DEVNAMES>) {
+		s/\r|\n//g;
+		local @l = split(/\t+/, $_);
+		$dscsi[$i]->{'make'} = $l[0];
+		$dscsi[$i]->{'model'} = $l[1];
+		$i++;
+		}
+	close(DEVNAMES);
+	$dscsi_mode = 1;
+	@dscsi = grep { $_->{'type'} == 0 } @dscsi;
+	}
+else {
+	# Check /proc/scsi/scsi for SCSI disk models
+	open(SCSI, "/proc/scsi/scsi");
+	local @lines = <SCSI>;
+	close(SCSI);
+	if ($lines[0] =~ /^Attached\s+domains/i) {
+		# New domains format
+		local $dscsi;
+		foreach (@lines) {
+			s/\s/ /g;
+			if (/Device:\s+(.*)(sd[a-z])\s+usage/) {
+				$dscsi = { 'dev' => $2 };
+				push(@dscsi, $dscsi);
+				}
+			elsif (/Device:/) {
+				$dscsi = undef;
+				}
+			elsif (/Vendor:\s+(\S+)\s+Model:\s+(\S+)/ && $dscsi) {
+				$dscsi->{'make'} = $1;
+				$dscsi->{'model'} = $2;
+				}
+			elsif (/Host:\s+scsi(\d+)\s+Channel:\s+(\d+)\s+Id:\s+(\d+)\s+Lun:\s+(\d+)/ && $dscsi) {
+				$dscsi->{'host'} = $1;
+				$dscsi->{'bus'} = $2;
+				$dscsi->{'target'} = $3;
+				$dscsi->{'lun'} = $4;
+				}
+			}
+		$dscsi_mode = 1;
+		}
+	else {
+		# Standard format
+		foreach (@lines) {
+			s/\s/ /g;
+			if (/^Host:/) {
+				push(@pscsi, $_);
+				}
+			elsif (/^\s+\S/ && @pscsi) {
+				$pscsi[$#pscsi] .= $_;
+				}
+			}
+		@pscsi = grep { /Type:\s+Direct-Access/i } @pscsi;
+		$dscsi_mode = 0;
 		}
 	}
-close(SCSI);
-@pscsi = grep { /Type:\s+Direct-Access/i } @pscsi;
 
 local (@disks, @devs, $d);
 if (open(PARTS, "/proc/partitions")) {
@@ -51,7 +110,7 @@ if (open(PARTS, "/proc/partitions")) {
 			# New or old style SCSI device
 			local $d = $1;
 			local ($host, $bus, $target, $lun) = ($2, $3, $4, $5);
-			if ($pscsi[$sc] =~ /USB-FDU/) {
+			if (!$dscsi_mode && $pscsi[$sc] =~ /USB-FDU/) {
 				# USB floppy with scsi emulation!
 				splice(@pscsi, $sc, 1);
 				next;
@@ -150,7 +209,9 @@ while(<FDISK>) {
 		if ($disk->{'device'} =~ /\/sd(\S)$/) {
 			# Old-style SCSI disk
 			$disk->{'desc'} = &text('select_device', 'SCSI',uc($1));
-			$disk->{'scsi'} = ord(uc($1))-65;
+			local ($dscsi) = grep { $_->{'dev'} eq "sd$1" } @dscsi;
+			$disk->{'scsi'} = $dscsi ? &indexof($dscsi, @dscsi)
+						 : ord(uc($1))-65;
 			$disk->{'type'} = 'scsi';
 			}
 		elsif ($disk->{'device'} =~ /\/hd(\S)$/) {
@@ -166,11 +227,22 @@ while(<FDISK>) {
 			$disk->{'lun'} = $5;
 			$disk->{'desc'} = &text('select_scsi',
 						"$2", "$3", "$4", "$5");
+			# Work out the SCSI index for this disk
 			local $j;
-			for($j=0; $j<@pscsi; $j++) {
-				if ($pscsi[$j] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i && $disk->{'host'} == $1 && $disk->{'target'} == $2) {
-					$disk->{'scsi'} = $j;
-					last;
+			if ($dscsi_mode) {
+				for($j=0; $j<@dscsi; $j++) {
+					if ($dscsi[$j]->{'host'} == $disk->{'host'} && $dscsi[$j]->{'bus'} == $disk->{'bus'} && $dscsi[$j]->{'target'} == $disk->{'target'} && $dscsi[$j]->{'lnun'} == $disk->{'lun'}) {
+						$disk->{'scsi'} = $j;
+						last;
+						}
+					}
+				}
+			else {
+				for($j=0; $j<@pscsi; $j++) {
+					if ($pscsi[$j] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i && $disk->{'host'} == $1 && $disk->{'target'} == $2) {
+						$disk->{'scsi'} = $j;
+						last;
+						}
 					}
 				}
 			$disk->{'type'} = 'scsi';
@@ -301,12 +373,21 @@ foreach $d (@disks) {
 foreach $d (@disks) {
 	if ($d->{'type'} eq 'scsi') {
 		local $s = $d->{'scsi'};
-		if ($pscsi[$s] =~ /Vendor:\s+(\S+).*Model:\s+(.*)\s+Rev:/i) {
-			$d->{'model'} = "$1 $2";
+		if ($dscsi_mode) {
+			# From other scsi files
+			$d->{'model'} = "$dscsi[$s]->{'make'} $dscsi[$s]->{'model'}";
+			$d->{'controller'} = $dscsi[$s]->{'host'};
+			$d->{'scsiid'} = $dscsi[$s]->{'target'};
 			}
-		if ($pscsi[$s] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i) {
-			$d->{'controller'} = int($1);
-			$d->{'scsiid'} = int($2);
+		else {
+			# From /proc/scsi/scsi lines
+			if ($pscsi[$s] =~ /Vendor:\s+(\S+).*Model:\s+(.*)\s+Rev:/i) {
+				$d->{'model'} = "$1 $2";
+				}
+			if ($pscsi[$s] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i) {
+				$d->{'controller'} = int($1);
+				$d->{'scsiid'} = int($2);
+				}
 			}
 		}
 	}
@@ -415,14 +496,16 @@ return $tags{$_[0]} ? $tags{$_[0]}
 # Given a partition tag, returns the filesystem type (assuming it is supported)
 sub conv_type
 {
+local @rv;
 if ($_[0] eq "4" || $_[0] eq "6" ||
-    $_[0] eq "1" || $_[0] eq "e") { $rv = "msdos"; }
-elsif ($_[0] eq "b" || $_[0] eq "c") { return "vfat"; }
-elsif ($_[0] eq "83") { $rv = "ext2"; }
-elsif ($_[0] eq "81") { $rv = "minix"; }
-else { return undef; }
-if (&has_command("mkfs.$rv")) { return $rv; }
-return undef;
+    $_[0] eq "1" || $_[0] eq "e") { @rv = ( "msdos" ); }
+elsif ($_[0] eq "b" || $_[0] eq "c") { @rv = ( "vfat" ); }
+elsif ($_[0] eq "83") { @rv = ( "ext2", "ext3" ); }
+elsif ($_[0] eq "81") { @rv = ( "minix" ); }
+else { return ( ); }
+local %supp = map { $_, 1 } &mount::list_fstypes();
+@rv = grep { $supp{$_} } @rv;
+return wantarray ? @rv : $rv[0];
 }
 
 # fstype_name(type)
@@ -870,7 +953,7 @@ else { return " $_[2] $in{$_[0]}"; }
 	 '3', 'XENIX usr',
 	 '4', 'FAT16 <32M',
 	 '6', 'FAT16',
-	 '7', 'HPFS/NTFS',
+	 '7', 'NTFS',
 	 '8', 'AIX',
 	 '9', 'AIX bootable',
 	 'a', 'OS/2 boot manager',
@@ -882,7 +965,7 @@ else { return " $_[2] $in{$_[0]}"; }
 	'12', 'Compaq diagnostic',
 	'14', 'Hidden FAT16 < 32M',
 	'16', 'Hidden FAT16',
-	'17', 'Hidden HPFS/NTFS',
+	'17', 'Hidden NTFS',
 	'18', 'AST Windows swapfile',
 	'1b', 'Hidden Win95 FAT (1b)',
 	'1c', 'Hidden Win95 FAT (1c)',
