@@ -1,10 +1,10 @@
 
-do './fdisk-lib.pl';
+do 'fdisk-lib.pl';
 
 sub feedback_files
 {
 return ( "/proc/partitions", "/proc/scsi/scsi",
-	 "fdisk -l" );
+	 "fdisk -l |" );
 }
 
 1;
