@@ -52,9 +52,10 @@ if ($config{'darwin_setup'}) {
 	}
 elsif (!$config{'init_base'}) {
 	# Add to the boot time rc script
-	open(LOCAL, ">>$config{'local_script'}");
-	print LOCAL "$config_directory/start >/dev/null 2>&1 </dev/null	# Start $ucproduct\n";
-	close(LOCAL);
+	$lref = &read_file_lines($config{'local_script'});
+	for($i=0; $i<@$lref && $lref->[$i] !~ /^exit\s/; $i++) { }
+	splice(@$lref, $i, 0, "$config_directory/start >/dev/null 2>&1 </dev/null # Start $ucproduct");
+	&flush_file_lines();
 	print STDERR "Added to bootup script $config{'local_script'}\n";
 	}
 else {
