@@ -13,6 +13,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 
 # runlevel_actions(level, S|K)
 # Return a list of actions started or stopped in some run-level, each in
@@ -520,6 +521,13 @@ if ($need_links) {
 		local $i;
 		for($i=0; $i<@$lref && $lref->[$i] !~ /^exit\s/; $i++) { }
 		splice(@$lref, $i, 0, "$fn start");
+		if ($config{'local_down'}) {
+			# Also add to shutdown script
+			$lref = &read_file_lines($config{'local_down'});
+			for($i=0; $i<@$lref &&
+				  $lref->[$i] !~ /^exit\s/; $i++) { }
+			splice(@$lref, $i, 0, "$fn stop");
+			}
 		&flush_file_lines();
 		}
 	}
@@ -567,12 +575,24 @@ else {
 	# Take out of rc.local file
 	local $lref = &read_file_lines($config{'local_script'});
 	local $cmd = "$module_config_directory/$_[0].sh start";
+	local $i;
 	for($i=0; $i<@$lref; $i++) {
 		if ($lref->[$i] eq $cmd) {
 			splice(@$lref, $i, 1);
 			last;
 			}
 		}
+	if ($config{'local_down'}) {
+		# Take out of shutdown script
+		$lref = &read_file_lines($config{'local_down'});
+		local $cmd = "$module_config_directory/$_[0].sh stop";
+		for($i=0; $i<@$lref; $i++) {
+			if ($lref->[$i] eq $cmd) {
+				splice(@$lref, $i, 1);
+				last;
+				}
+			}
+		}
 	&flush_file_lines();
 	}
 }
