@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './inittab-lib.pl';
+do 'inittab-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -21,6 +21,9 @@ elsif ($action eq 'create') {
 elsif ($action eq 'delete') {
 	return &text('log_delete', "<tt>$object</tt>");
 	}
+elsif ($action eq 'apply') {
+	return $text{'log_apply'};
+	}
 else {
 	return undef;
 	}
