@@ -2,6 +2,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 
 $google_host = "www.google.com";
 $google_port = 80;
@@ -22,5 +23,14 @@ if ($perl_doc && $] >= 5.006) {
 	$perl_doc = "$perl_doc -U";
 	}
 
+# set_manpath(extra)
+sub set_manpath
+{
+$ENV{'MANPATH'} = join(":", split(/:/, $config{'man_dir'}),
+			    split(/:/, $_[0]),
+			    "/usr/man",
+			    "/usr/share/man");
+}
+
 1;
 
