@@ -2,6 +2,10 @@
 # Mount table functions for freebsd
 
 $uname_release = `uname -r`;
+if (&has_command("mount_smbfs")) {
+	$smbfs_support = 1;
+	$nsmb_conf = "/etc/nsmb.conf";
+	}
 
 # Return information about a filesystem, in the form:
 #  directory, device, type, options, fsck_order, mount_at_boot
@@ -20,7 +24,23 @@ while(<FSTAB>) {
 	if (!/\S/) { next; }
 	@p = split(/\s+/, $_);
 	if ($p[2] eq "proc" || $p[2] eq "procfs") { $p[2] = $p[0] = "proc"; }
-	if ($p[2] eq "swap") { $p[1] = "swap"; }
+	elsif ($p[2] eq "swap") { $p[1] = "swap"; }
+	elsif ($p[2] eq "smbfs") {
+		# Need to get nsmb.conf options, covert share and extract user
+		$p[0] = lc($p[0]);
+		local $noptions = &read_nsmb($p[0]);
+		local %options;
+		&parse_options($p[3]);
+		if ($p[0] =~ /^\/\/(\S+)\@(\S+)\/(.*)$/) {
+			$p[0] = "\\\\$2\\$3";
+			$options{'user'} = $1;
+			}
+		elsif ($p[0] =~ /\/\/(\S+)\/(.*)$/) {
+			$p[0] = "\\\\$1\\$2";
+			}
+		%options = ( %options, %$noptions );
+		$p[3] = &join_options();
+		}
 	$rv[$i] = [ $p[1], $p[0], $p[2] ];
 	$rv[$i]->[5] = "yes";
 	@o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
@@ -42,12 +62,37 @@ return @rv;
 # Add a new entry to the fstab file, old or new automounter file
 sub create_mount
 {
-local(@mlist, @amd, $_); local($opts);
+local(@mlist, @amd, $_, $opts);
 
-# Adding a normal mount to the fstab file
 open(FSTAB, ">> $config{'fstab_file'}");
-print FSTAB "$_[1]  $_[0]  $_[2]";
-$opts = $_[3] eq "-" ? "" : $_[3];
+&print_mount_line(@_);
+close(FSTAB);
+}
+
+# print_mount_line(directory, device, type, options, fsck_order, mount_at_boot)
+sub print_mount_line
+{
+if ($_[2] eq "smbfs") {
+	# Adding an SMB mount, which needs special handling
+	local %options;
+	&parse_options("smbfs", $_[3]);
+	local $share;
+	if ($options{'user'} && $_[1] =~ /^\\\\(.*)\\(.*)$/) {
+		$share = "//$options{'user'}\@$1/$2";
+		}
+	else {
+		($share = $_[1]) =~ s/\\/\//g;
+		}
+	print FSTAB "$share  $_[0]  $_[2]";
+	local $roptions = &update_nsmb($share, \%options);
+	delete($roptions->{'user'});
+	$opts = &join_options($_[2], $roptions);
+	}
+else {
+	# Adding a normal mount to the fstab file
+	print FSTAB "$_[1]  $_[0]  $_[2]";
+	$opts = $_[3] eq "-" ? "" : $_[3];
+	}
 if ($_[5] eq "no") {
 	$opts = join(',' , (split(/,/ , $opts) , "noauto"));
 	}
@@ -75,15 +120,7 @@ foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print FSTAB "$_[2]  $_[1]  $_[3]";
-		$opts = $_[4] eq "-" ? "" : $_[4];
-		if ($_[6] eq "no") {
-			$opts = join(',' , (split(/,/ , $opts) , "noauto"));
-			}
-		if ($opts eq "") { print FSTAB "  defaults"; }
-		else { print FSTAB "  $opts"; }
-		print FSTAB "  0  ";
-		print FSTAB $_[5] eq "-" ? "0\n" : "$_[5]\n";
+		&print_mount_line(@_[1..$#_]);
 		}
 	else { print FSTAB $_,"\n"; }
 	}
@@ -126,13 +163,32 @@ sub list_mounted
 {
 # get the list of mounted filesystems
 local(@rv, $_);
-local $cmd = $uname_release =~ /^4\.[0-9]/ ? "./freebsd-mounts-4" :
-	     $uname_release =~ /^3\.[1-9]/ ? "./freebsd-mounts-3" :
-				             "./freebsd-mounts-2";
-open(CMD, "$cmd |");
+local $cmd = $uname_release =~ /^5\.[0-9]/ ? "freebsd-mounts-5" :
+	     $uname_release =~ /^4\.[0-9]/ ? "freebsd-mounts-4" :
+	     $uname_release =~ /^3\.[1-9]/ ? "freebsd-mounts-3" :
+				             "freebsd-mounts-2";
+&compile_program($cmd, '.*86');
+open(CMD, "$module_config_directory/$cmd |");
 while(<CMD>) {
 	local @p = split(/\t/, $_);
 	if ($p[2] eq "procfs" || $p[1] eq "procfs") { $p[1] = $p[2] = "proc"; }
+	elsif ($p[2] eq "mfs") { $p[1] =~ s/:.*$//; }
+	elsif ($p[2] eq "smbfs") {
+		# Need to get nsmb.conf options, covert share and extract user
+		$p[1] = lc($p[1]);
+		local $noptions = &read_nsmb($p[1]);
+		local %options;
+		&parse_options($p[3]);
+		if ($p[1] =~ /^\/\/(\S+)\@(\S+)\/(.*)$/) {
+			$p[1] = "\\\\$2\\$3";
+			$options{'user'} = $1;
+			}
+		elsif ($p[1] =~ /\/\/(\S+)\/(.*)$/) {
+			$p[1] = "\\\\$1\\$2";
+			}
+		%options = ( %options, %$noptions );
+		$p[3] = &join_options();
+		}
 	push(@rv, \@p);
 	}
 close(CMD);
@@ -158,10 +214,39 @@ if ($_[2] eq "swap") {
 	$out = &backquote_logged("swapon $_[1] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
+elsif ($_[2] eq "smbfs") {
+	# Need special handling for SMB mounts
+	local %options;
+	&parse_options($_[2], $_[3]);
+	local $share;
+	if ($options{'user'} && $_[1] =~ /^\\\\(.*)\\(.*)$/) {
+		$share = "//$options{'user'}\@$1/$2";
+		}
+	else {
+		($share = $_[1]) =~ s/\\/\//g;
+		}
+	local $roptions = &update_nsmb($share, \%options);
+	delete($roptions->{'user'});
+	local $opts = &join_options($_[2], $roptions);
+	$opts = $opts ne "-" ? " -o $opts" : "";
+	&foreign_require("proc", "proc-lib.pl");
+	local ($fh, $fpid) = &proc::pty_process_exec_logged(
+		"mount -t $_[2] $opts $share $_[0]");
+	local $got;
+	local $rv = &wait_for($fh, "Password:");
+	$got .= $wait_for_input;
+	if ($rv == 0) {
+		print $fh $options{'nsmb_password'},"\n";
+		}
+	$rv = &wait_for($fh);
+	$got .= $wait_for_input;
+	close($fh);
+	if ($? || $got =~ /failed|error|syserr|usage:/i) { return "<pre>$out</pre>"; }
+	}
 else {
-	# some disk-based filesystem
-	$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
-	$opts = join(',', grep { !/quota/ } split(/,/, $opts));
+	# Can use mount command for this filesystem
+	$opts = $_[3] eq "-" ? "" :
+			"-o ".join(',', grep { !/quota/ } split(/,/, $_[3]));
 	$out = &backquote_logged("mount -t $_[2] $opts $_[1] $_[0] 2>&1");
 	if ($?) { return "<pre>$out</pre>"; }
 	}
@@ -240,6 +325,7 @@ sub list_fstypes
 local @rv = ("ufs", "nfs", "cd9660", "msdos", "swap");
 push(@rv, "ext2fs") if (&has_command("mount_ext2fs"));
 push(@rv, "ntfs") if (&has_command("mount_ntfs"));
+push(@rv, "smbfs") if ($smbfs_support);
 return @rv;
 }
 
@@ -256,7 +342,8 @@ local(%fsmap);
 	  "ext2fs","Linux Filesystem",
 	  "ntfs","Windows NT Filesystem",
 	  "swap","Virtual Memory",
-	  "proc","Process Image Filesystem");
+	  "proc","Process Image Filesystem",
+	  "smbfs","Windows Networking Filesystem");
 return $config{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
 }
 
@@ -265,7 +352,7 @@ return $config{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
 # Returns 1 if filesystems of this type can be mounted multiple times, 0 if not
 sub multiple_mount
 {
-return $_[0] eq "nfs";
+return $_[0] eq "nfs" || $_[0] eq "smbfs";
 }
 
 
@@ -285,6 +372,18 @@ if ($_[0] eq "nfs") {
 	&nfs_export_chooser_button("nfs_host", "nfs_dir");
 	print "</td> </tr>\n";
 	}
+elsif ($_[0] eq "smbfs") {
+	# SMB mount from some server and share
+	$_[1] =~ /^\\\\(.*)\\(.*)$/;
+	print "<tr> <td><b>$text{'linux_smbserver'}</b></td>\n";
+	print "<td><input name=smbfs_server value=\"$1\" size=20>\n";
+	&smb_server_chooser_button("smbfs_server");
+	print "</td>\n";
+	print "<td><b>$text{'linux_smbshare'}</b></td>\n";
+	print "<td><input name=smbfs_share value=\"$2\" size=20>\n";
+	&smb_share_chooser_button("smbfs_server", "smbfs_share");
+	print "</td> </tr>\n";
+	}
 else {
 	if ($_[0] eq "swap") {
 		# Swap file or device
@@ -296,10 +395,10 @@ else {
 			&fstype_name($_[0]);
 		}
 	print "<td colspan=3>\n";
-	if ($_[1] =~ /^\/dev\/wd(\d)s(\d)([a-z]*)$/) {
+	if ($_[1] =~ /^\/dev\/ad(\d)s(\d)([a-z]*)$/) {
 		$disk_dev = 0; $ide_t = $1; $ide_s = $2; $ide_p = $3;
 		}
-	elsif ($_[1] =~ /^\/dev\/sd(\d)s(\d)([a-z]*)$/) {
+	elsif ($_[1] =~ /^\/dev\/da(\d)s(\d)([a-z]*)$/) {
 		$disk_dev = 1; $scsi_t = $1; $scsi_s = $2; $scsi_p = $3;
 		}
 	else { $disk_dev = 2; }
@@ -407,7 +506,7 @@ if ($_[0] eq "ufs") {
 	printf "<input type=radio name=ufs_groupquota value=1 %s> Enabled\n",
 		defined($options{'groupquota'}) && $options{'groupquota'} eq ""
 			? "checked" : "";
-	printf "<input type=radio name=ufs_groupquota value=2 %s>\n";
+	printf "<input type=radio name=ufs_groupquota value=2 %s>\n",
 		$options{'groupquota'} ? "checked" : "";
 	print "Enabled, use file\n";
 	printf "<input name=ufs_groupquota_file size=30 value=\"%s\">\n",
@@ -548,6 +647,30 @@ elsif ($_[0] eq "swap") {
 	# Swap has no options..
 	print "<tr> <td><i>No Options Available</i></td> </tr>\n";
 	}
+elsif ($_[0] eq "smbfs") {
+	# SMBFS has some special options
+	print "<tr> <td><b>$text{'linux_username'}</b></td>\n";
+	printf "<td><input name=smbfs_user size=15 value=\"%s\"></td>\n",
+		$options{"user"};
+
+	print "<td><b>$text{'linux_password'}</b></td>\n";
+	printf "<td><input type=password name=smbfs_password size=15 value=\"%s\"></td> </tr>\n",
+		$options{"nsmb_password"};
+
+	print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
+	printf "<td colspan=3><input type=radio name=smbfs_workgroup_def value=1 %s> $text{'linux_auto'}\n",
+		defined($options{"nsmb_workgroup"}) ? "" : "checked";
+	printf "<input type=radio name=smbfs_workgroup_def value=0 %s>\n",
+		defined($options{"nsmb_workgroup"}) ? "checked" : "";
+	print "<input size=10 name=smbfs_workgroup value=\"$options{'nsmb_workgroup'}\"></td> </tr>\n";
+
+	print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
+	printf "<td colspan=3><input type=radio name=smbfs_addr_def value=1 %s> %s\n",
+		defined($options{"nsmb_addr"}) ? "" : "checked", $text{'linux_auto'};
+	printf "<input type=radio name=smbfs_addr_def value=0 %s>\n",
+		defined($options{"nsmb_addr"}) ? "checked" : "";
+	print "<input size=30 name=smbfs_addr value=\"$options{'nsmb_addr'}\"></td> </tr>\n";
+	}
 }
 
 
@@ -608,10 +731,16 @@ if ($_[0] eq "nfs") {
 		&error("NFS Error - $mout");
 		}
 	# It worked! unmount
-	`umount $temp`;
+	system("umount $temp");
 	rmdir($temp);
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
+elsif ($_[0] eq "smbfs") {
+	# A windows server filesystem .. check the server and share
+	$in{'smbfs_server'} =~ /\S/ || &error($text{'linux_eserver'});
+	$in{'smbfs_share'} =~ /\S/ || &error($text{'linux_eshare'});
+	return "\\\\".lc($in{'smbfs_server'})."\\".lc($in{'smbfs_share'});
+	}
 else {
 	# This is some kind of disk-based filesystem.. get the device name
 	if ($in{'disk_dev'} == 0) {
@@ -621,7 +750,7 @@ else {
 			&error("'$in{ide_s}' is not a valid slice number");
 		$in{'ide_p'} =~ /^[a-z]*$/ ||
 			&error("'$in{ide_p}' is not a valid partition letter");
-		$dv = "/dev/wd$in{ide_t}s$in{ide_s}$in{ide_p}";
+		$dv = "/dev/ad$in{ide_t}s$in{ide_s}$in{ide_p}";
 		}
 	elsif ($in{'disk_dev'} == 1) {
 		$in{'scsi_t'} =~ /^\d+$/ ||
@@ -630,7 +759,7 @@ else {
 			&error("'$in{scsi_s}' is not a valid slice number");
 		$in{'scsi_p'} =~ /^[a-z]*$/ ||
 			&error("'$in{scsi_p}' is not a valid partition letter");
-		$dv = "/dev/sd$in{scsi_t}s$in{scsi_s}$in{scsi_p}";
+		$dv = "/dev/da$in{scsi_t}s$in{scsi_s}$in{scsi_p}";
 		}
 	else {
 		$dv = $in{'dev_path'};
@@ -767,13 +896,32 @@ elsif ($_[0] eq "ntfs") {
 	delete($options{"-i"});
 	$options{"-i"} = '' if ($in{'ntfs_i'});
 	}
+elsif ($_[0] eq "smbfs") {
+	# Parse SMBFS options
+	delete($options{'user'});
+	$options{'user'} = $in{'smbfs_user'} if ($in{'smbfs_user'});
+
+	delete($options{'nsmb_password'});
+	$options{'nsmb_password'} = $in{'smbfs_password'}
+		if ($in{'smbfs_password'});
+
+	delete($options{'nsmb_addr'});
+	if (!$in{"smbfs_addr_def"}) {
+		&check_ipaddress($in{"smbfs_addr"}) ||
+			&error($text{'freebsd_eaddr'});
+		$options{'nsmb_addr'} = $in{"smbfs_addr"};
+		}
+
+	delete($options{'nsmb_workgroup'});
+	if (!$in{"smbfs_workgroup_def"}) {
+		$in{"smbfs_workgroup"} =~ /^\S+$/ ||
+			&error($text{'freebsd_eworkgroup'});
+		$options{'nsmb_workgroup'} = $in{"smbfs_workgroup"};
+		}
+	}
 
 # Return options string
-foreach $k (keys %options) {
-	if ($options{$k} eq "") { push(@rv, $k); }
-	else { push(@rv, "$k=$options{$k}"); }
-	}
-return @rv ? join(',' , @rv) : "-";
+return &join_options();
 }
 
 # create_swap(file, size, units)
@@ -824,7 +972,119 @@ return $_[0];
 
 sub files_to_lock
 {
-return ( $config{'fstab_file'} );
+return ( $config{'fstab_file'}, $nsmb_conf );
+}
+
+# get_nsmb_conf(server, share, user)
+# Finds a single nsmb.conf section
+sub get_nsmb_conf
+{
+local $conf;
+local $insection = 0;
+local $lnum = 0;
+open(CONF, $nsmb_conf);
+while(<CONF>) {
+	s/\r|\n//g;
+	s/^\s*#.*$//;
+	if (/^\s*\[([^:]+):([^:]+):([^:]+)\]/ &&
+	    lc($1) eq lc($_[0]) && lc($2) eq lc($_[2]) && lc($3) eq lc($_[1])) {
+		# Start of section
+		$insection = 1;
+		$conf = { 'line' => $lnum,
+			  'eline' => $lnum,
+			  'server' => lc($_[0]),
+			  'user' => lc($_[2]),
+			  'share' => lc($_[1]) };
+		}
+	elsif (/^\s*\[.*\]/) {
+		# Start of another section
+		$insection = 0;
+		}
+	elsif (/^\s*(\S+)\s*=\s*(\S+)/ && $insection) {
+		$conf->{'values'}->{lc($1)} = $2;
+		$conf->{'eline'} = $lnum;
+		}
+	$lnum++;
+	}
+close(CONF);
+return $conf;
+}
+
+# save_nsmb_conf(&conf)
+# Updates or creates a single nsmb.conf section
+sub save_nsmb_conf
+{
+local $lref = &read_file_lines($nsmb_conf);
+local @lines = ( "[$_[0]->{'server'}:$_[0]->{'user'}:$_[0]->{'share'}]" );
+foreach $k (keys %{$_[0]->{'values'}}) {
+	push(@lines, $k."=".$_[0]->{'values'}->{$k});
+	}
+if (defined($_[0]->{'line'})) {
+	# Modifying
+	splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
+	       @lines);
+	}
+else {
+	# Adding
+	push(@$lref, @lines);
+	}
+&flush_file_lines();
+}
+
+# update_nsmb(share, &options)
+sub update_nsmb
+{
+local ($server, $share, $user);
+if ($_[0] =~ /^[\\\/]{2}(\S+)\@(\S+)[\\\/](\S+)$/) {
+	($user, $server, $share) = ($1, $2, $3);
+	}
+elsif ($_[0] =~ /^[\\\/]{2}(\S+)[\\\/](\S+)$/) {
+	($user, $server, $share) = ("root", $1, $2);
+	}
+else {
+	&error("Invalid share $_[0]");
+	}
+local $conf = &get_nsmb_conf($server, $share, $user);
+$conf ||= { "server" => $server,
+	    "share" => $share,
+	    "user" => $user };
+$conf->{'values'} = { };
+local %others;
+foreach $k (keys %{$_[1]}) {
+	if ($k =~ /^nsmb_(.*)$/) {
+		$conf->{'values'}->{$1} = $_[1]->{$k};
+		}
+	else {
+		$others{$k} = $_[1]->{$k};
+		}
+	}
+&save_nsmb_conf($conf);
+return \%others;
+}
+
+# read_nsmb(share)
+# Returns a hash reference containing options for some share
+sub read_nsmb
+{
+local ($server, $share, $user);
+if ($_[0] =~ /^[\\\/]{2}(\S+)\@(\S+)[\\\/](\S+)$/) {
+	($user, $server, $share) = ($1, $2, $3);
+	}
+elsif ($_[0] =~ /^[\\\/]{2}(\S+)[\\\/](\S+)$/) {
+	($user, $server, $share) = ("root", $1, $2);
+	}
+else {
+	&error("Invalid share $_[0]");
+	}
+local $conf = &get_nsmb_conf($server, $share, $user);
+if ($conf) {
+	local (%rv, $k);
+	foreach $k (keys %{$conf->{'values'}}) {
+		$rv{"nsmb_".$k} = $conf->{'values'}->{$k};
+		}
+	return \%rv;
+	}
+return undef;
 }
 
 1;
