@@ -244,7 +244,7 @@ local(%fsmap);
 	  "cdfs","ISO9660 CD-ROM",
 	  "lofs","Loopback Filesystem",
 	  "swapfs","Filesystem Swap Space",
-	  "swap","Swap Space",
+	  "swap","Virtual Memory",
 	  "autofs","Automounter Filesystem");
 return $config{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
 }
