@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './mount-lib.pl';
+do 'mount-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params, [long])
 # Converts logged information from this module into human-readable form
