@@ -4,6 +4,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 %access = &get_module_acl();
 
 # get_mount(directory|'swap', device)
@@ -145,7 +146,12 @@ if (&has_command($config{'smbclient_path'})) {
 }
 
 # Include the correct OS-specific functions file
-do "$gconfig{'os_type'}-lib.pl";
+if ($gconfig{'os_type'} =~ /^\S+\-linux$/) {
+	do "linux-lib.pl";
+	}
+else {
+	do "$gconfig{'os_type'}-lib.pl";
+	}
 
 # can_edit_fs(dir, device, type, options)
 # Returns 1 if a filesystem can be edited, 0 otherwise
