@@ -126,7 +126,8 @@ sub list_mounted
 {
 # get the list of mounted filesystems
 local(@rv, $_);
-local $cmd = "netbsd-mounts";
+local $cmd = $uname_release =~ /^2\.[0-9]/ ? "netbsd-mounts-2"
+					   : "netbsd-mounts";
 &compile_program($cmd);
 open(CMD, "$module_config_directory/$cmd |");
 while(<CMD>) {
@@ -247,7 +248,7 @@ return @rv;
 sub fstype_name
 {
 local(%fsmap);
-%fsmap = ("ffs", "OpenBSD Unix Filesystem",
+%fsmap = ("ffs", "NetBSD Unix Filesystem",
 	  "nfs","Network Filesystem",
 	  "cd9660","ISO9660 CD-ROM",
 	  "msdos","MS-DOS Filesystem",
