@@ -162,7 +162,7 @@ sub list_mounted
 {
 return @list_mounted_cache if (@list_mounted_cache);
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `swap -l`)) {
+foreach (split(/\n/, `swap -l 2>/dev/null`)) {
 	if (/^(\/\S+)\s+/) { push(@rv, [ "swap", $1, "swap", "-" ]); }
 	}
 foreach (split(/\n/, `cat /etc/mnttab`)) {
@@ -671,7 +671,7 @@ if ($_[0] eq "nfs") {
 		defined($options{"vers"}) ? "" : "checked";
 	printf "<input type=radio name=nfs_vers_def value=0 %s>\n",
 		defined($options{"vers"}) ? "checked" : "";
-	print "<input size=1 name=nfs_vers value=$options{vers}></td> </tr>\n";
+	print "<input size=2 name=nfs_vers value=$options{vers}></td> </tr>\n";
 
 	print "<tr> <td><b>$text{'solaris_proto'}</b></td>\n";
 	print "<td nowrap><select name=proto>\n";
@@ -765,52 +765,53 @@ if ($_[0] eq "ufs") {
 		}
 	print "</select></td>\n";
 
-	print "<td><b>$text{'solaris_toosoon'}</b></td>\n";
-	$options{toosoon} =~ /([0-9]+)([A-z])/;
-	print "<td nowrap><input size=5 name=ufs_toosoon_time value=$1>\n";
-	print "<select name=ufs_toosoon_units>\n";
-	foreach $u ('s', 'm', 'h', 'd', 'w', 'y') {
-		printf "<option value=%s %s> %s\n",
-			$u, $2 eq $u ? "selected" : "", $text{"solaris_time_$u"};
-		}
-	print "</select></td> </tr>\n";
-
 	if ($gconfig{'os_version'} >= 7) {
-		print "<tr> <td><b>$text{'solaris_noatime'}</b></td> <td>\n";
+		print "<td><b>$text{'solaris_noatime'}</b></td> <td>\n";
 		if ($gconfig{'os_version'} >= 8) {
 			printf "<input type=radio name=ufs_noatime value=0 %s> $text{'solaris_immed'}\n",
 				defined($options{'nodfratime'}) ? "checked" : "";
 			printf "<input type=radio name=ufs_noatime value=1 %s> $text{'solaris_defer'}\n",
 				!defined($options{'nodfratime'}) &&
 				!defined($options{'noatime'}) ? "checked" : "";
-			printf "<input type=radio name=ufs_noatime value=2 %s> $text{'no'}</td>\n",
+			printf "<input type=radio name=ufs_noatime value=2 %s> $text{'no'}</td> </tr>\n",
 				defined($options{'noatime'}) ? "checked" : "";
 			}
 		else {
 			printf "<input type=radio name=ufs_noatime value=0 %s> $text{'yes'}\n",
 				defined($options{'noatime'}) ? "" : "checked";
-			printf "<input type=radio name=ufs_noatime value=1 %s> $text{'no'}</td>\n",
+			printf "<input type=radio name=ufs_noatime value=1 %s> $text{'no'}</td> </tr>\n",
 				defined($options{'noatime'}) ? "checked" : "";
 			}
 
-		print "<td><b>$text{'solaris_force'}</b></td> <td>\n";
+		print "<tr> <td><b>$text{'solaris_force'}</b></td> <td>\n";
 		printf "<input type=radio name=ufs_force value=1 %s> $text{'yes'}\n",
 			defined($options{'forcedirectio'}) ? "checked" : "";
-		printf "<input type=radio name=ufs_force value=0 %s> $text{'no'}</td> </tr>\n",
+		printf "<input type=radio name=ufs_force value=0 %s> $text{'no'}</td>\n",
 			defined($options{'forcedirectio'}) ? "" : "checked";
 
-		print "<tr> <td><b>$text{'solaris_nolarge'}</td> <td>\n";
+		print "<td><b>$text{'solaris_nolarge'}</td> <td>\n";
 		printf "<input type=radio name=ufs_nolarge value=0 %s> $text{'yes'}\n",
 			defined($options{'nolargefiles'}) ? "" : "checked";
-		printf "<input type=radio name=ufs_nolarge value=1 %s> $text{'no'}</td>\n",
+		printf "<input type=radio name=ufs_nolarge value=1 %s> $text{'no'}</td> </tr>\n",
 			defined($options{'nolargefiles'}) ? "checked" : "";
 
-		print "<td><b>$text{'solaris_logging'}</td> <td>\n";
+		print "<tr> <td><b>$text{'solaris_logging'}</td> <td>\n";
 		printf "<input type=radio name=ufs_logging value=1 %s> $text{'yes'}\n",
 			defined($options{'logging'}) ? "checked" : "";
 		printf "<input type=radio name=ufs_logging value=0 %s> $text{'no'}</td> </tr>\n",
 			defined($options{'logging'}) ? "" : "checked";
 		}
+	else {
+		print "<td><b>$text{'solaris_toosoon'}</b></td>\n";
+		$options{toosoon} =~ /([0-9]+)([A-z])/;
+		print "<td nowrap><input size=5 name=ufs_toosoon_time value='$1'>\n";
+		print "<select name=ufs_toosoon_units>\n";
+		foreach $u ('s', 'm', 'h', 'd', 'w', 'y') {
+			printf "<option value=%s %s> %s\n",
+				$u, $2 eq $u ? "selected" : "", $text{"solaris_time_$u"};
+			}
+		print "</select></td> </tr>\n";
+		}
 	}
 if ($_[0] eq "hsfs") {
 	# Solaris hsfs is used for CDROMs
@@ -1322,7 +1323,7 @@ else {
 
 # Check the filesystem type
 $out = `fstyp $_[0] 2>&1`;
-if ($out =~ /^([A-z0-9]+)\n$/) {
+if ($out =~ /^([A-Za-z0-9]+)\n$/) {
 	if ($1 eq $_[1]) { return; }
 	else {
 		# Wrong filesystem type
@@ -1427,11 +1428,6 @@ elsif ($_[0] eq "ufs") {
 		$options{"onerror"} = $in{ufs_onerror};
 		}
 
-	delete($options{"toosoon"});
-	if ($in{ufs_toosoon_time}) {
-		$options{"toosoon"} = $in{ufs_toosoon_time}.
-				      $in{ufs_toosoon_units};
-		}
 	if ($gconfig{'os_version'} >= 7) {
 		if ($gconfig{'os_version'} >= 8) {
 			delete($options{'noatime'});
@@ -1454,6 +1450,13 @@ elsif ($_[0] eq "ufs") {
 		delete($options{'logging'}); delete($options{'nologging'});
 		$options{'logging'} = "" if ($in{'ufs_logging'});
 		}
+	else {
+		delete($options{"toosoon"});
+		if ($in{ufs_toosoon_time}) {
+			$options{"toosoon"} = $in{ufs_toosoon_time}.
+					      $in{ufs_toosoon_units};
+			}
+		}
 	}
 elsif ($_[0] eq "lofs") {
 	# Loopback has no options to parse
