@@ -153,7 +153,7 @@ close(AUTOTAB);
 sub list_mounted
 {
 local(@rv, @p, $_, $i, $r);
-foreach (split(/\n/, `swap -l`)) {
+foreach (split(/\n/, `swap -l 2>/dev/null`)) {
 	if (/^(\/\S+)\s+/) { push(@rv, [ "swap", $1, "swap", "-" ]); }
 	}
 foreach (split(/\n/, `cat /etc/mnttab`)) {
@@ -258,8 +258,9 @@ else {
 		 (defined($options{'readwrite'}) ? "-S " : "").
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
-		$rv = &system_logged("rumba \"$shar\" $_[0] $opts >/tmp/$$.rumba 2>&1 </dev/null");
-		$out = `cat /tmp/$$.rumba`; unlink("/tmp/$$.rumba");
+		local $rtemp = &tempname();
+		$rv = system("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
+		$out = `cat $rtemp`; unlink($rtemp);
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
 	else {
@@ -910,9 +911,9 @@ if ($_[0] eq "tmpfs") {
 	($tmpsz = $options{size}) =~ s/[A-z]+$//g;
 	print "<input name=tmpfs_size size=6 value=\"$tmpsz\">\n";
 	print "<select name=tmpfs_unit>\n";
-	printf "<option value=m %s> Mb\n",
+	printf "<option value=m %s> MB\n",
 		$options{"size"} =~ /m$/ ? "selected" : "";
-	printf "<option value=k %s> Kb\n",
+	printf "<option value=k %s> kB\n",
 		$options{"size"} =~ /k$/ ? "selected" : "";
 	printf "<option value=b %s> bytes\n",
 		$options{"size"} !~ /(k|m)$/ ? "selected" : "";
