@@ -270,7 +270,7 @@ if ($remote_user && @_ > 1) {
 			    $gaccess{'feedback'}) {
 				print qq~<td><img src=$gconfig{'webprefix'}/images/top_bar/top_sep.jpg width=12 height=32></td>~;
 				}
-			print qq~<td width="84" nowrap><a href='$logout'><img src="$gconfig{'webprefix'}/images/top_bar/$loicon" height="31" width=$lowidth border="0" alt="$lotext"></a></td>~;
+			print qq~<td width="84" nowrap><a href='$gconfig{'webprefix'}$logout'><img src="$gconfig{'webprefix'}/images/top_bar/$loicon" height="31" width=$lowidth border="0" alt="$lotext"></a></td>~;
 			}
 		}
 	print qq~<td width="3" nowrap>
@@ -396,7 +396,7 @@ if (@_ > 1 && !$one && $remote_user && !$notabs) {
                 print qq~</a></center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~; 
               } else {
-               print qq~<td nowrap><a href=$gconfig{'webprefix'}/?cat=$c><center><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t"><br>~;
 
             &chop_font;
 
@@ -456,8 +456,9 @@ if (@_ > 1 && (!$_[5] || $ENV{'HTTP_WEBMIN_SERVERS'})) {
 		local $idx = $module_info{'index_link'};
 		local $mi = $module_index_link ||
 			    $module_name ? "/$module_name/$idx" : "/";
+		local $mt = $module_index_name || $text{'header_module'};
 		&tab_start; print "<a href=\"$gconfig{'webprefix'}$mi\">",
-			    "$text{'header_module'}</a>"; &tab_end;
+			    "$mt</a>"; &tab_end;
 		}
 	if (ref($_[2]) eq "ARRAY" && !$ENV{'ANONYMOUS_USER'}) {
 		&tab_start; print &hlink($text{'header_help'}, $_[2]->[0], $_[2]->[1]); &tab_end;
@@ -467,7 +468,7 @@ if (@_ > 1 && (!$_[5] || $ENV{'HTTP_WEBMIN_SERVERS'})) {
 		}
 	if ($_[3]) {
 		if (!$access{'noconfig'}) {
-			&tab_start; print "<a href=\"/config.cgi?$module_name\">",
+			&tab_start; print "<a href=\"$gconfig{'webprefix'}/config.cgi?$module_name\">",
 			      $text{'header_config'},"</a>"; &tab_end;
 			}
 		}
@@ -496,11 +497,6 @@ print qq~
     if (!$_[5]) {
 	    # Show page title in tab
 	    local $title = $_[0];
-	    $title =~ s/&auml;//g;
-	    $title =~ s/&ouml;//g;
-	    $title =~ s/&uuml;//g;
-	    $title =~ s/&nbsp;/ /g;
-
 	    print "<p><table border=0 cellpadding=0 cellspacing=0 width=95% align=center><tr><td><table border=0 cellpadding=0 cellspacing=0 height=20><tr>\n";
 	    print "<td bgcolor=#bae3ff>",
 	      "<img src=$gconfig{'webprefix'}/images/tabs/blue_left.jpg width=13 height=22 ",
@@ -552,7 +548,7 @@ if ($theme_no_table) {
 	print "</ul>\n";
 	}
 elsif (@header_arguments > 1 && !$header_arguments[5]) {
-	print "</table></table><br>\n";
+	print "<br>\n</table></table><br>\n";
 	}
 
 print "<table border=0 width=100% align=center cellspacing=0 cellpadding=0><tr><td>\n";
@@ -569,6 +565,7 @@ for($i=0; $i+1<@_; $i+=2) {
     elsif ($url =~ /^\?/ && $module_name) {
         $url = "/$module_name/$url";
         }
+    $url = "$gconfig{'webprefix'}$url" if ($url =~ /^\//);
     if ($i == 0) {
         print "&nbsp;<a href=\"$url\"><img alt=\"<-\" align=middle border=0 src=$gconfig{'webprefix'}/images/arrow.jpg></a>\n";
         }
@@ -594,7 +591,7 @@ sub chop_font {
 if (!$current_lang_info->{'titles'} || $gconfig{'texttitles'}) {
 	print $t;
 } else {
-        foreach $l (split(//, $t)) {
+        foreach $l (split(//, &entities_to_ascii($t))) {
             $ll = ord($l);
 	    local $gif;
             if ($ll > 127 && $current_lang_info->{'charset'}) {
@@ -633,5 +630,46 @@ sub tab_end {
     </td>~;
 }
 
+sub theme_ui_post_header
+{
+local ($text) = @_;
+local $rv;
+if (defined($text)) {
+	$rv .= "<center><font size=+1>$text</font></center>\n";
+	}
+if ($main::printed_unbuffered_header) {
+	$rv .= "<hr>\n";
+	}
+elsif (!defined($text)) {
+	$rv .= "<br>\n";
+	}
+return $rv;
+}
+
+sub theme_ui_pre_footer
+{
+local $rv;
+if ($main::printed_unbuffered_header) {
+	$rv .= "<hr>\n";
+	}
+return $rv;
+}
+
+sub theme_error
+{
+&header($text{'error'}, "");
+print "<br>\n";
+print "<h3>",($main::whatfailed ? "$main::whatfailed : " : ""),@_,"</h3>\n";
+&footer();
+}
+
+sub theme_ui_print_unbuffered_header
+{
+$| = 1;
+$theme_no_table = 1;
+$main::printed_unbuffered_header = 1;
+&ui_print_header(@_);
+}
+
 1;
 
