@@ -5,16 +5,20 @@ require 'net-lib.pl';
 # Output HTML for editing security options for the net module
 sub acl_security_form
 {
-print "<tr> <td><b>$text{'acl_ifcs'}</b></td> <td colspan=3>\n";
+print "<tr> <td valign=top><b>$text{'acl_ifcs'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=ifcs value=2 %s> $text{'yes'}\n",
 	$_[0]->{'ifcs'} == 2 ? "checked" : "";
 printf "<input type=radio name=ifcs value=1 %s> $text{'acl_view'}\n",
 	$_[0]->{'ifcs'} == 1 ? "checked" : "";
-printf "<input type=radio name=ifcs value=0 %s> $text{'no'}\n",
+printf "<input type=radio name=ifcs value=0 %s> $text{'no'}<br>\n",
 	$_[0]->{'ifcs'} ? "" : "checked";
 printf "<input type=radio name=ifcs value=3 %s> $text{'acl_ifcs_only'}\n",
 	$_[0]->{'ifcs'} == 3 ? "checked" : "";
-print "<input name=interfaces size=30 value='".$_[0]->{'interfaces'}."'> ",
+print "<input name=interfaces3 size=30 value='".$_[0]->{'interfaces'}."'> ",
+	&interfaces_chooser_button("interfaces", 1),"<br>\n";
+printf "<input type=radio name=ifcs value=4 %s> $text{'acl_ifcs_ex'}\n",
+	$_[0]->{'ifcs'} == 4 ? "checked" : "";
+print "<input name=interfaces4 size=30 value='".$_[0]->{'interfaces'}."'> ",
 	&interfaces_chooser_button("interfaces", 1),"</td> </tr>\n";
 
 print "<tr> <td><b>$text{'acl_routes'}</b></td> <td colspan=3>\n";
@@ -56,7 +60,8 @@ $_[0]->{'ifcs'} = $in{'ifcs'};
 $_[0]->{'routes'} = $in{'routes'};
 $_[0]->{'dns'} = $in{'dns'};
 $_[0]->{'hosts'} = $in{'hosts'};
-$_[0]->{'interfaces'} = $in{'interfaces'};
+$_[0]->{'interfaces'} = $in{'ifcs'} == 3 ? $in{'interfaces3'} :
+			$in{'ifcs'} == 4 ? $in{'interfaces4'} : undef;
 $_[0]->{'apply'} = $in{'apply'};
 }
 
