@@ -45,6 +45,7 @@ foreach $iface (@ifaces) {
 		$cfg->{'bootp'} = ($method eq 'bootp');
 		$cfg->{'edit'} = ($cfg->{'name'} !~ /^ppp|lo/);
 		$cfg->{'index'} = scalar(@rv);	
+		$cfg->{'file'} = $network_interfaces_config;
 		push(@rv, $cfg);
 		}
 	}
@@ -79,6 +80,7 @@ else {
 		}
 	}
 my @autos = get_auto_defs();
+print STDERR "old autos = ",join(" ", @autos),"\n";
 my $amode = $gconfig{'os_version'} > 3 || scalar(@autos);
 if (!$cfg->{'up'} && !$amode) { push(@options, ['noauto', '']); }
 
@@ -107,6 +109,7 @@ if ($amode) {
 	else {
 		@autos = grep { $_ ne $cfg->{'fullname'} } @autos;
 		}
+	print STDERR "new autos = ",join(" ", @autos),"\n";
 	&modify_auto_defs(@autos);
 	}
 }
@@ -118,7 +121,7 @@ sub delete_interface
 my $cfg = $_[0];
 delete_interface_def($cfg->{'fullname'}, 'inet');
 my @autos = get_auto_defs();
-if ($gconfig{'os_version'} > 3 || scalar(@autos)) {
+if ($gconfig{'os_version'} >= 3 || scalar(@autos)) {
 	@autos = grep { $_ ne $cfg->{'fullname'} } @autos;
 	&modify_auto_defs(@autos);
 	}
@@ -166,6 +169,7 @@ while(<SWITCH>) {
 		}
 	}
 close(SWITCH);
+$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
 return $dns;
 }
 
@@ -309,12 +313,18 @@ else {
 &write_env_file($network_config, \%conf);
 }
 
+sub routing_config_files
+{
+return ( $network_interfaces_config );
+}
+
 # show default router and device
 sub routing_input
 {
 local ($addr, $router) = &get_default_gateway();
 local @ifaces = &get_interface_defs();
 
+# Show default gateway
 print "<tr> <td><b>$text{'routes_default'}</b></td>\n";
 printf "<td><input type=radio name=gateway_def value=1 %s> %s\n",
 	$addr ? '' : 'checked', $text{'routes_none'};
@@ -329,6 +339,59 @@ foreach $iface (@ifaces) {
 		$router eq $iface->[0] ? 'selected' : '', $iface->[0];
 	}
 print "</select></td> </tr>\n";
+
+# Get static routes
+local ($d, @st, @hr);
+foreach $d (@ifaces) {
+	local ($name, $addrfam, $method, $options) = @$d;
+	local $o;
+	local $onum = -1;
+	foreach $o (@$options) {
+		$onum++;
+		next if ($o->[0] ne "up");
+		if ($o->[1] =~ /^ip\s+route\s+add\s+([0-9\.]+)\/(\d+)\s+dev\s+(\S+)/) {
+			push(@hr, [ $name, $1, &prefix_to_mask($2) ]);
+			}
+		elsif ($o->[1] =~ /^ip\s+route\s+add\s+([0-9\.]+)\/(\d+)\s+via\s+(\S+)/) {
+			push(@st, [ $name, $1, &prefix_to_mask($2), $3 ]);
+			}
+		}
+	}
+
+# Show static routes via gateways
+print "<tr> <td valign=top><b>$text{'routes_static'}</b></td>\n";
+print "<td><table border>\n";
+print "<tr $tb> <td><b>$text{'routes_ifc'}</b></td> ",
+      "<td><b>$text{'routes_net'}</b></td> ",
+      "<td><b>$text{'routes_mask'}</b></td> ",
+      "<td><b>$text{'routes_gateway'}</b></td> </tr>\n";
+local $i;
+for($i=0; $i<=@st; $i++) {
+	local $st = $st[$i];
+	print "<tr $cb>\n";
+	print "<td><input name=dev_$i size=6 value='$st->[0]'></td>\n";
+	print "<td><input name=net_$i size=15 value='$st->[1]'></td>\n";
+	print "<td><input name=netmask_$i size=15 value='$st->[2]'></td>\n";
+	print "<td><input name=gw_$i size=15 value='$st->[3]'></td>\n";
+	print "</tr>\n";
+	}
+print "</table></td> </tr>\n";
+
+# Show static host routes
+print "<tr> <td valign=top><b>$text{'routes_local'}</b></td>\n";
+print "<td><table border>\n";
+print "<tr $tb> <td><b>$text{'routes_ifc'}</b></td> ",
+      "<td><b>$text{'routes_net'}</b></td> ",
+      "<td><b>$text{'routes_mask'}</b></td> </tr>\n";
+for($i=0; $i<=@hr; $i++) {
+	local $st = $hr[$i];
+	print "<tr $cb>\n";
+	print "<td><input name=ldev_$i size=6 value='$st->[0]'></td>\n";
+	print "<td><input name=lnet_$i size=15 value='$st->[1]'></td>\n";
+	print "<td><input name=lnetmask_$i size=15 value='$st->[2]'></td>\n";
+	print "</tr>\n";
+	}
+print "</table></td> </tr>\n";
 }
 
 sub parse_routing
@@ -341,6 +404,48 @@ if (!$in{'gateway_def'}) {
 	$dev = $in{'gatewaydev'};
 	}
 &set_default_gateway($gw, $dev);
+
+# Parse static and local routes
+local %st;
+local $i;
+local $dev;
+for($i=0; defined($dev = $in{"dev_$i"}); $i++) {
+	next if (!$dev);
+	local $net = $in{"net_$i"};
+	local $netmask = $in{"netmask_$i"};
+	local $gw = $in{"gw_$i"};
+	$dev =~ /^\S+$/ || &error(&text('routes_edevice', $dev));
+	gethostbyname($net) || &error(&text('routes_enet', $net));
+	&check_ipaddress($netmask) || &error(&text('routes_emask', $netmask));
+	gethostbyname($gw) || &error(&text('routes_egateway', $gw));
+	local $prefix = &mask_to_prefix($netmask);
+	push(@{$st{$dev}}, [ "up", "ip route add $net/$prefix via $gw" ]);
+	}
+local %hr;
+for($i=0; defined($dev = $in{"ldev_$i"}); $i++) {
+	local $net = $in{"lnet_$i"};
+	local $netmask = $in{"lnetmask_$i"};
+	next if (!$dev && !$net);
+	$dev =~ /^\S+$/ || &error(&text('routes_edevice', $dev));
+	gethostbyname($net) || $net =~ /^(\S+)\/(\d+)$/ && gethostbyname($1) ||
+		&error(&text('routes_enet', $net));
+	&check_ipaddress($netmask) || &error(&text('routes_emask', $netmask));
+	local $prefix = &mask_to_prefix($netmask);
+	push(@{$hr{$dev}}, [ "up", "ip route add $net/$prefix dev $dev" ]);
+	}
+
+# Replace old routing directives
+local @ifaces = &get_interface_defs();
+foreach $iface (@ifaces) {
+	local @o = @{$iface->[3]};
+	@o = grep { $_->[0] ne "up" ||
+		    $_->[1] !~ /^ip\s+route\s+add/ } @o;
+	push(@o, @{$st{$iface->[0]}});
+	push(@o, @{$hr{$iface->[0]}});
+	$iface->[3] = \@o;
+	&modify_interface_def($iface->[0], $iface->[1], $iface->[2],
+			      $iface->[3], 0);
+	}
 }
 
 
@@ -442,6 +547,8 @@ return @rv;
 sub modify_auto_defs
 {
 local $lref = &read_file_lines($network_interfaces_config);
+local $i;
+local $found;
 for($i=0; $i<@$lref; $i++) {
 	local $l = $lref->[$i];
 	$l =~ s/\r|\n//g;
