@@ -181,6 +181,7 @@ foreach $r (keys %rc) {
 	$ifc{'up'} = 1;
 	$ifc{'edit'} = 1;
 	$ifc{'index'} = scalar(@rv);
+	$ifc{'file'} = "/etc/rc.conf";
 	push(@rv, \%ifc);
 	}
 return @rv;
@@ -258,7 +259,7 @@ return	$_[0] =~ /^tun/ ? "Loopback tunnel" :
 	$_[0] =~ /^ppp/ ? "PPP" :
 	$_[0] =~ /^lo/ ? "Loopback" :
 	$_[0] =~ /^ar/ ? "Arnet" :
-	$_[0] =~ /^bge/ ? "Gigabit ethernet" :
+	$_[0] =~ /^(bge|em)/ ? "Gigabit ethernet" :
 	$_[0] =~ /^(ax|mx|pn|rl|tx|wb)/ ? "Fast ethernet" :
 	$_[0] =~ /^(cs|dc|de|ed|el|ex|fe|fxp|ie|le|lnc|tl|vr|vx|xl|ze|zp)/ ? "Ethernet" : $text{'ifcs_unknown'};
 }
@@ -312,6 +313,7 @@ while(<HOST>) {
 	push(@{$dns->{'order'}}, $_) if (/\S/);
 	}
 close(HOST);
+$dns->{'files'} = [ "/etc/resolv.conf", "/etc/host.conf" ];
 return $dns;
 }
 
@@ -400,6 +402,11 @@ sub save_hostname
 &unlock_file("/etc/rc.conf");
 }
 
+sub routing_config_files
+{
+return ( "/etc/defaults/rc.conf", "/etc/rc.conf" );
+}
+
 sub routing_input
 {
 local %rc = &get_rc_conf();
