@@ -33,6 +33,7 @@ while($f = readdir(CONF)) {
 	$b->{'dhcp'} = $conf{'DYNAMIC'} eq 'dhcp';
 	$b->{'edit'} = ($b->{'name'} !~ /^ppp|plip/);
 	$b->{'index'} = scalar(@rv);
+	$b->{'file'} = "$net_scripts_dir/$f";
 	push(@rv, $b);
 	}
 closedir(CONF);
@@ -123,6 +124,7 @@ while(<SWITCH>) {
 		}
 	}
 close(SWITCH);
+$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
 return $dns;
 }
 
@@ -262,6 +264,12 @@ else {
 # XXX need to update nis.conf
 }
 
+sub routing_config_files
+{
+return ( $network_config,
+	 map { $_->{'file'} } &boot_interfaces() );
+}
+
 sub routing_input
 {
 local (%conf, %ifc, $f, $gateway, $gatewaydev);
