@@ -168,6 +168,7 @@ sub boot_interfaces
 	      $ifc{'up'} = 1;
 	      $ifc{'edit'} = 1;
 	      $ifc{'index'} = scalar(@rv);
+	      $ifc{'file'} = $r;
 	      push(@rv, \%ifc);
 	  }
 	  close( IF_FILE );
@@ -330,6 +331,7 @@ sub get_dns_config
       }
       close(RESOLV);
       
+      $dns->{'files'} = [ "/etc/resolv.conf" ];
       return $dns;
   }
 
@@ -451,6 +453,11 @@ sub read_routing {
     $rd = "NO" if( $rd eq '' );
 }
 
+sub routing_config_files
+{
+return ( "/etc/mygate", "/etc/sysctl.conf", "/etc/rc.conf" );
+}
+
 sub routing_input
   {
       &read_routing;
