@@ -19,6 +19,7 @@ foreach $l (@lines) {
 	local %ifc;
 	$l =~ /^([^:\s]+):/; $ifc{'name'} = $1;
 	$l =~ /^(\S+):/; $ifc{'fullname'} = $1;
+	if ($l =~ /inet6\s+(\S+)/) { next; }	# skip IPv6 for now
 	if ($l =~ /^(\S+):(\d+):\s/) { $ifc{'virtual'} = $2; }
 	if ($l =~ /inet\s+(\S+)/) { $ifc{'address'} = $1; }
 	if ($l =~ /netmask\s+(\S+)/) { $ifc{'netmask'} = &parse_hex($1); }
@@ -42,12 +43,16 @@ return @rv;
 sub activate_interface
 {
 local $a = $_[0];
-if ($a->{'virtual'} eq "") {
-	local $out = &backquote_logged("ifconfig $a->{'name'} plumb 2>&1");
-	if ($out) { &error(&text('aifc_eexist', $a->{'name'})); }
-	}
-elsif ($gconfig{'os_version'} >= 8) {
-	&system_logged("ifconfig $a->{'name'}:$a->{'virtual'} plumb >/dev/null 2>&1");
+local @active = &active_interfaces();
+local ($already) = grep { $_->{'fullname'} eq $_[0]->{'fullname'} } @active;
+if (!$already) {
+	if ($a->{'virtual'} eq "") {
+		local $out = &backquote_logged("ifconfig $a->{'name'} plumb 2>&1");
+		if ($out) { &error(&text('aifc_eexist', $a->{'name'})); }
+		}
+	elsif ($gconfig{'os_version'} >= 8) {
+		&system_logged("ifconfig $a->{'name'}:$a->{'virtual'} plumb >/dev/null 2>&1");
+		}
 	}
 local $cmd = "ifconfig $a->{'name'}";
 if ($a->{'virtual'} ne "") { $cmd .= ":$a->{'virtual'}"; }
@@ -109,13 +114,14 @@ while(<MASK>) {
 close(MASK);
 opendir(ETC, "/etc");
 while($f = readdir(ETC)) {
-	if ($f =~ /^hostname.(\S+):(\d+)$/ || $f =~ /^hostname.(\S+)/) {
+	if ($f =~ /^hostname\.(\S+):(\d+)$/ || $f =~ /^hostname\.(\S+)/) {
 		local %ifc;
 		$ifc{'fullname'} = $ifc{'name'} = $1;
 		$ifc{'virtual'} = $2 if (defined($2));
 		$ifc{'fullname'} .= ":$2" if (defined($2));
 		$ifc{'index'} = scalar(@rv);
 		$ifc{'edit'}++;
+		$ifc{'file'} = "/etc/$f";
 		open(FILE, "/etc/$f");
 		chop($ifc{'address'} = <FILE>);
 		close(FILE);
@@ -240,6 +246,7 @@ while(<SWITCH>) {
 		}
 	}
 close(SWITCH);
+$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
 return $dns;
 }
 
@@ -342,6 +349,11 @@ return &get_system_hostname();
 sub save_hostname
 {
 &system_logged("hostname $_[0] >/dev/null 2>&1");
+if (-r "/etc/nodename") {
+	open(NODENAME, ">/etc/nodename");
+	print NODENAME $_[0],"\n";
+	close(NODENAME);
+	}
 }
 
 # get_domainname()
@@ -364,16 +376,29 @@ if ($_[0]) {
 else { unlink("/etc/defaultdomain"); }
 }
 
-sub routing_input
+sub routing_config_files
 {
-# show default router(s) input
-local(@defrt);
+return ( "/etc/defaultrouter", "/etc/notrouter", "/etc/gateways" );
+}
+
+# get_defaultrouters()
+# Returns a list of all default routers
+sub get_defaultrouters
+{
+local @defrt;
 open(DEFRT, "/etc/defaultrouter");
 while(<DEFRT>) {
 	s/#.*$//g;
 	if (/(\S+)/) { push(@defrt, $1); }
 	}
 close(DEFRT);
+return @defrt;
+}
+
+sub routing_input
+{
+# show default router(s) input
+local @defrt = &get_defaultrouters();
 print "<tr> <td valign=top><b>$text{'routes_defaults'}</b></td>\n";
 print "<td><textarea name=defrt rows=3 cols=40>",
 	join("\n", @defrt),"</textarea></td> </tr>\n";
@@ -442,5 +467,80 @@ return ( @f, "/etc/netmasks", "/etc/resolv.conf", "/etc/nsswitch.conf",
 	 "/etc/defaultrouter", "/etc/notrouter", "/etc/gateways" );
 }
 
+# get_default_gateway()
+# Returns the default gateway IP (if one is set) boot time
+# settings.
+sub get_default_gateway
+{
+local @defrt = &get_defaultrouters();
+return @defrt ? ( $defrt[0] ) : ( );
+}
+
+# set_default_gateway(gateway, device)
+# Sets the default gateway to the given IP accessible via the given device,
+# in the boot time settings.
+sub set_default_gateway
+{
+&lock_file("/etc/defaultrouter");
+if ($_[0]) {
+	open(DEF, ">/etc/defaultrouter");
+	print DEF $_[0],"\n";
+	close(DEF);
+	}
+else {
+	unlink("/etc/defaultrouter");
+	}
+&unlock_file("/etc/defaultrouter");
+}
+
+# list_routes()
+# Returns a list of active routes
+sub list_routes
+{
+local @rv;
+open(ROUTES, "netstat -rn |");
+while(<ROUTES>) {
+	s/\s+$//;
+	if (/^([0-9\.]+|default)\s+([0-9\.]+)\s+\S+\s+\S+\s+\S+(\s+(\S+))?$/) {
+		push(@rv, { 'dest' => $1 eq "default" ? "0.0.0.0" : $1,
+			    'gateway' => $2,
+			    'iface' => $4 });
+		}
+	}
+close(ROUTES);
+return @rv;
+}
+
+# apply_network()
+# Apply the interface and routing settings, by activating all interfaces and
+# adding the default route
+sub apply_network
+{
+local (%done, $b, $a);
+foreach $b (&boot_interfaces()) {
+	&activate_interface($b) if (!$b->{'dhcp'} && !$b->{'bootp'});
+	$done{$b->{'fullname'}}++;
+	}
+foreach $a (&active_interfaces()) {
+	if (!$done{$a->{'fullname'}}) {
+		&deactive_interface($a);
+		}
+	}
+local @infile = &get_defaultrouters();
+local @routes = &list_routes();
+local @inmem = map { $_->{'gateway'} }
+		   grep { $_->{'dest'} eq "0.0.0.0" } @routes;
+if (join(" ", @infile) ne join(" ", @inmem)) {
+	# Fix up default routes
+	local $r;
+	foreach $r (@inmem) {
+		system("route delete default $r >/dev/null 2>&1");
+		}
+	foreach $r (@infile) {
+		system("route add default $r >/dev/null 2>&1");
+		}
+	}
+}
+
 1;
 
