@@ -19,6 +19,7 @@ push(@rv, { 'fullname' => 'lo',
 	    'netmask' => '255.0.0.0',
 	    'broadcast' => '127.255.255.255',
 	    'edit' => 0,
+	    'file' => $rc_config,
 	    'index' => scalar(@rv) });
 local @nc = split(/\s+/, $rc->{'NETCONFIG'}->{'value'});
 foreach $nc (@nc) {
@@ -60,6 +61,7 @@ foreach $nc (@nc) {
 		$b->{'edit'} = 1;
 		$b->{'index'} = scalar(@rv);
 		$b->{'nc'} = $nc;
+		$b->{'file'} => $rc_config,
 		push(@rv, $b);
 		}
 	}
@@ -212,6 +214,7 @@ while(<SWITCH>) {
 		}
 	}
 close(SWITCH);
+$dns->{'files'} = [ $rc_config, "/etc/nsswitch.conf" ];
 return $dns;
 }
 
@@ -343,6 +346,11 @@ local $rc = &parse_rc_config();
 system("SuSEconfig -quick >/dev/null 2>&1");
 }
 
+sub routing_config_files
+{
+return ( $route_conf, $rc_config );
+}
+
 sub routing_input
 {
 # read route.conf
