@@ -1,3 +1,4 @@
+#!/usr/local/bin/perl
 # perlpath.pl
 # This script gets run only from setup.sh in order to replace the 
 # #!/usr/local/bin/perl line at the start of scripts with the real path to perl
@@ -16,9 +17,9 @@ foreach $f (@files) {
 	open(IN, $f);
 	@lines = <IN>;
 	close(IN);
-	if ($lines[0] =~ /^#!\/.*perl/) {
+	if ($lines[0] =~ /^#!\/\S*perl\S*(.*)/) {
 		open(OUT, "> $f");
-		print OUT "#!$ppath\n";
+		print OUT "#!$ppath$1\n";
 		for($i=1; $i<@lines; $i++) {
 			print OUT $lines[$i];
 			}
