@@ -1,5 +1,5 @@
 
-require './proc-lib.pl';
+require 'proc-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the proc module
