@@ -1,6 +1,8 @@
 # linux-lib.pl
 # Functions for parsing linux ps output
 
+use Time::Local;
+
 sub list_processes
 {
 local($pcmd, $line, $i, %pidmap, @plist, $dummy, @w, $_);
@@ -229,6 +231,88 @@ close(LOAD);
 return ( $load[0], $load[1], $load[2] );
 }
 
+$has_trace_command = &has_command("strace");
+
+# open_process_trace(pid, [&syscalls])
+# Starts tracing on some process, and returns a trace object
+sub open_process_trace
+{
+local $fh = time().$$;
+local $sc;
+if (@{$_[1]}) {
+	$sc = "-e trace=".join(",", @{$_[1]});
+	}
+local $tpid = open($fh, "strace -t -p $_[0] $sc 2>&1 |");
+$line = <$fh>;
+return { 'pid' => $_[0],
+	 'tpid' => $tpid,
+	 'fh' => $fh };
+}
+
+# close_process_trace(&trace)
+# Halts tracing on some trace object
+sub close_process_trace
+{
+kill('TERM', $_[0]->{'tpid'}) if ($_[0]->{'tpid'});
+close($_[0]->{'fh'});
+}
+
+# read_process_trace(&trace)
+# Returns an action structure representing one action by traced process, or
+# undef if an error occurred
+sub read_process_trace
+{
+local $fh = $_[0]->{'fh'};
+local @tm = localtime(time());
+while(1) {
+	local $line = <$fh>;
+	return undef if (!$line);
+	if ($line =~ /^(\d+):(\d+):(\d+)\s+([^\(]+)\((.*)\)\s*=\s*(\-?\d+|\?)/) {
+		local $tm = timelocal($3, $2, $1, $tm[3], $tm[4], $tm[5]);
+		local $action = { 'time' => $tm,
+				  'call' => $4,
+				  'rv' => $6 eq "?" ? undef : $6 };
+		local $args = $5;
+		local @args;
+		while(1) {
+			if ($args =~ /^[ ,]*(\{[^}]*\})(.*)$/) {
+				# A structure in { }
+				push(@args, $1);
+				$args = $2;
+				}
+			elsif ($args =~ /^[ ,]*"([^"]*)"\.*(.*)$/) {
+				# A quoted string
+				push(@args, $1);
+				$args = $2;
+				}
+			elsif ($args =~ /^[ ,]*\[([^\]]*)\](.*)$/) {
+				# A square-bracket number
+				push(@args, $1);
+				$args = $2;
+				}
+			elsif ($args =~ /^[ ,]*\<([^\>]*)\>(.*)$/) {
+				# An angle-bracketed string
+				push(@args, $1);
+				$args = $2;
+				}
+			elsif ($args =~ /[ ,]*([^, ]+)(.*)$/) {
+				# Just a number
+				push(@args, $1);
+				$args = $2;
+				}
+			else {
+				last;
+				}
+			}
+		if ($args[$#args] eq $action->{'rv'}) {
+			pop(@args);	# last arg is same as return value?
+			}
+		$action->{'args'} = \@args;
+		return $action;
+		}
+	}
+}
+
 foreach $ia (keys %text) {
 	if ($ia =~ /^linux(_\S+)/) {
 		$info_arg_map{$1} = $text{$ia};
