@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './raid-lib.pl';
+do 'raid-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -10,7 +10,7 @@ sub parse_webmin_log
 local ($user, $script, $action, $type, $object, $p) = @_;
 $object = &html_escape($object);
 if ($action eq 'create') {
-	return &text('log_create', $text{$p->{'level'}}, "<tt>$object</tt>");
+	return &text('log_create', $p->{'level'} eq 'linear' ? $text{$p->{'level'}} : $text{'raid'.$p->{'level'}}, "<tt>$object</tt>");
 	}
 elsif ($action eq 'stop') {
 	return &text('log_stop', "<tt>$object</tt>");
@@ -22,7 +22,13 @@ elsif ($action eq 'delete') {
 	return &text('log_delete', "<tt>$object</tt>");
 	}
 elsif ($action eq 'mkfs') {
-	return &text('log_mkfs', "<tt>EXT2</tt>", "<tt>$object</tt>");
+	return &text('log_mkfs', "<tt>$p->{'fs'}</tt>", "<tt>$object</tt>");
+	}
+elsif ($action eq 'add') {
+	return &text('log_add', "<tt>$object</tt>", "<tt>$p->{'disk'}</tt>");
+	}
+elsif ($action eq 'remove') {
+	return &text('log_remove', "<tt>$object</tt>", "<tt>$p->{'disk'}</tt>");
 	}
 else {
 	return undef;
