@@ -7,8 +7,9 @@ do './web-lib.pl';
 &init_config();
 
 foreach $m (&get_all_module_infos()) {
+	$mdir = &module_root_directory($m->{'dir'});
 	if (&check_os_support($m) &&
-	    -r "$root_directory/$m->{'dir'}/uninstall.pl") {
+	    -r "$mdir/uninstall.pl") {
 		# Call this module's uninstall function
 		eval {
 			&foreign_require($m->{'dir'}, "uninstall.pl");
