@@ -1,5 +1,5 @@
 
-do './servers-lib.pl';
+do 'servers-lib.pl';
 
 sub feedback_files
 {
