@@ -3,6 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 %access = &get_module_acl();
 
 sub list_servers
@@ -135,6 +136,7 @@ else {
 
 @server_types = ( [ 'caldera', 'OpenLinux', 'open-linux' ],
 		  [ 'redhat', 'Redhat Linux', 'redhat-linux' ],
+		  [ 'fedora', 'Fedora Linux', undef, 'Fedora' ],
 		  [ 'suse', 'SuSE Linux', 'suse-linux' ],
 		  [ 'debian', 'Debian Linux', 'debian-linux' ],
 		  [ 'mandrake', 'Mandrake Linux', 'mandrake-linux' ],
@@ -155,7 +157,8 @@ sub this_server
 {
 local $type = 'unknown';
 foreach $s (@server_types) {
-	if ($s->[2] && $gconfig{'os_type'} =~ /^$s->[2]$/) {
+	if ($s->[2] && $gconfig{'os_type'} =~ /^$s->[2]$/ ||
+	    $s->[3] && $gconfig{'real_os_type'} =~ /$s->[3]/) {
 		$type = $s->[0];
 		last;
 		}
