@@ -69,6 +69,12 @@ else
 fi
 cd "$wadir"
 
+# Work out perl library path
+PERLLIB=$wadir
+if [ "$perllib" != "" ]; then
+	PERLLIB="$PERLLIB:$perllib"
+fi
+
 # Validate source directory
 allmods=`cd "$srcdir"; echo */module.info | sed -e 's/\/module.info//g'`
 if [ "$allmods" = "" ]; then
@@ -477,12 +483,21 @@ else
 	else
 		echo "session=1" >> $cfile
 	fi
+	if [ "$pam" != "" ]; then
+		echo "pam=$pam" >> $cfile
+	fi
+
+	md5pass=`$perl -e 'print crypt("test", "\\$1\\$A9wB3O18\\$zaZgqrEmb9VNltWTL454R/") eq "\\$1\\$A9wB3O18\\$zaZgqrEmb9VNltWTL454R/" ? "1\n" : "0\n"'`
 
 	ufile=$config_dir/miniserv.users
 	if [ "$crypt" != "" ]; then
 		echo "$login:$crypt:0" > $ufile
 	else
-		$perl -e 'print "$ARGV[0]:",crypt($ARGV[1], "XX"),":0\n"' "$login" "$password" > $ufile
+		if [ "$md5pass" = "1" ]; then
+			$perl -e 'print "$ARGV[0]:",crypt($ARGV[1], "\$1\$XXXXXXXX"),":0\n"' "$login" "$password" > $ufile
+		else
+			$perl -e 'print "$ARGV[0]:",crypt($ARGV[1], "XX"),":0\n"' "$login" "$password" > $ufile
+		fi
 	fi
 	chmod 600 $ufile
 	echo "userfile=$ufile" >> $cfile
@@ -552,7 +567,7 @@ echo "export LANG" >>$config_dir/start
 echo "#PERLIO=:raw" >>$config_dir/start
 echo "unset PERLIO" >>$config_dir/start
 echo "export PERLIO" >>$config_dir/start
-echo "PERLLIB=$wadir" >>$config_dir/start
+echo "PERLLIB=$PERLLIB" >>$config_dir/start
 echo "export PERLLIB" >>$config_dir/start
 uname -a | grep -i 'HP/*UX' >/dev/null
 if [ $? = "0" ]; then
@@ -626,6 +641,11 @@ if [ "$?" != "0" ]; then
 	echo passwd_mode=0 >> $config_dir/miniserv.conf
 fi
 
+# If Perl crypt supports MD5, then make it the default
+if [ "$md5pass" = "1" ]; then
+	echo md5pass=1 >> $config_dir/config
+fi
+
 # Set a special theme if none was set before
 if [ "$theme" = "" ]; then
 	theme=`cat "$wadir/defaulttheme" 2>/dev/null`
@@ -650,6 +670,12 @@ if [ "$makeboot" = "1" ]; then
 	echo ""
 fi
 
+# If password delays are not specifically disabled, enable them
+grep passdelay= $config_dir/miniserv.conf >/dev/null
+if [ "$?" != "0" ]; then
+	echo passdelay=1 >> $config_dir/miniserv.conf
+fi
+
 if [ "$nouninstall" = "" ]; then
 	echo "Creating uninstall script $config_dir/uninstall.sh .."
 	cat >$config_dir/uninstall.sh <<EOF
@@ -704,10 +730,17 @@ else
 	rm -f $config_dir/install-dir
 fi
 
-echo "Running postinstall scripts .."
-(cd "$wadir" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/run-postinstalls.pl")
-echo "..done"
-echo ""
+if [ "$nopostinstall" = "" ]; then
+	echo "Running postinstall scripts .."
+	(cd "$wadir" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/run-postinstalls.pl")
+	echo "..done"
+	echo ""
+fi
+
+# Run package-defined post-install script
+if [ -r "$srcdir/setup-post.sh" ]; then
+	. "$srcdir/setup-post.sh"
+fi
 
 if [ "$nostart" = "" ]; then
 	if [ "$inetd" != "1" ]; then
@@ -754,8 +787,4 @@ if [ "$oldwadir" != "$wadir" -a "$upgrading" = 1 -a "$deletedold" != 1 ]; then
 	echo ""
 fi
 
-# Run package-defined post-install script
-if [ -r "$srcdir/setup-post.sh" ]; then
-	. "$srcdir/setup-post.sh"
-fi
 
