@@ -1,5 +1,5 @@
 
-do './shell-lib.pl';
+do 'shell-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the shell module
