@@ -10,11 +10,15 @@ sub acl_security_form
 	"</tr><tr>",
 	    "<td><b>", $text{ 'acl_hw' }, "</b></td>",
 	    "<td><input type=radio name=hwdate value=0 ", $_[0] -> { 'hwdate' } == 0 ? "checked" : "", ">", $text{ 'acl_yes' }, " <input type=radio name=hwdate value=1 ", $_[0] -> { 'hwdate' } == 1 ? "checked" : "", ">", $text{ 'acl_no' },
-	"</tr>" );
+	"</tr><tr>",
+	    "<td><b>", $text{ 'acl_timezone' }, "</b></td>",
+	    "<td><input type=radio name=timezone value=1 ", $_[0] -> { 'timezone' } == 1 ? "checked" : "", ">", $text{ 'acl_yes' }, " <input type=radio name=timezone value=0 ", $_[0] -> { 'timezone' } == 0 ? "checked" : "", ">", $text{ 'acl_no' },
+	"</tr>\n");
 }
 
 sub acl_security_save
 {
     $_[0] -> { 'sysdate' } = $in{ 'sysdate' };
     $_[0] -> { 'hwdate' } = $in{ 'hwdate' };
+    $_[0] -> { 'timezone' } = $in{ 'timezone' };
 }
