@@ -6,7 +6,7 @@
 # (/etc/master.passwd) and 2 for SysV (/etc/passwd & /etc/shadow)
 sub passfiles_type
 {
-return 2;
+return &password_file($config{'shadow_file'}) ? 2 : 0;
 }
 
 # groupfiles_type()
