@@ -1,8 +1,12 @@
 # user-lib.pl
 # Common functions for Unix user management
+#	XXX re-factor select all / invert functions
+#	XXX group mass deletion
+#	XXX locking and logging
 
 do '../web-lib.pl';
 &init_config();
+do '../ui-lib.pl';
 do "$gconfig{'os_type'}-lib.pl";
 
 @random_password_chars = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9' );
@@ -352,10 +356,10 @@ else {
 		# update AIX security user file as well..
 		# use chuser command because it's easier than working
 		# with the complexity issues of the file.
-		$_[1]->{'expire'} = 0 if (! $_[1]->{'expire'}); 
-		$_[1]->{'min'} = 0 if (! $_[1]->{'min'}); 
-		$_[1]->{'max'} = 0 if (! $_[1]->{'max'}); 
-		$_[1]->{'warn'} = 0 if (! $_[1]->{'warn'}); 
+		$_[1]->{'expire'} = '' if (! $_[1]->{'expire'}); 
+		$_[1]->{'min'} = '' if (! $_[1]->{'min'}); 
+		$_[1]->{'max'} = '' if (! $_[1]->{'max'}); 
+		$_[1]->{'warn'} = '' if (! $_[1]->{'warn'}); 
 		&system_logged("chuser expires=$_[1]->{'expire'} minage=$_[1]->{'min'} maxage=$_[1]->{'max'} pwdwarntime=$_[1]->{'warn'} $_[1]->{'user'}");
 		}
 	}
@@ -676,8 +680,9 @@ sub other_modules
 local($m, %minfo);
 local $func = shift(@_);
 foreach $m (&get_all_module_infos()) {
+	local $mdir = &module_root_directory($m->{'dir'});
 	if (&check_os_support($m) &&
-	    -r "$root_directory/$m->{'dir'}/useradmin_update.pl") {
+	    -r "$mdir/useradmin_update.pl") {
 		&foreign_require($m->{'dir'}, "useradmin_update.pl");
 		local $pkg = $m->{'dir'};
 		$pkg =~ s/[^A-Za-z0-9]/_/g;
@@ -876,6 +881,7 @@ else { return endpwent(); }
 sub my_getpwnam
 {
 if ($config{'from_files'}) {
+	local $u;
 	foreach $u (&list_users()) {
 		return &pw_user_rv($u, wantarray, 'uid')
 			if ($u->{'user'} eq $_[0]);
@@ -943,6 +949,7 @@ else { return endgrent(); }
 sub my_getgrnam
 {
 if ($config{'from_files'}) {
+	local $g;
 	foreach $g (&list_groups()) {
 		return &gr_group_rv($g, wantarray, 'gid')
 			if ($g->{'group'} eq $_[0]);
@@ -1087,7 +1094,7 @@ return undef;
 }
 
 # can_use_group(&acl, group)
-# Returns 1 if some group can be used as a primary secondary, 0 if not
+# Returns 1 if some group can be used as a primary or secondary, 0 if not
 sub can_use_group
 {
 return 1 if ($_[0]->{'ugroups'} eq '*');
@@ -1645,4 +1652,86 @@ sub berkeley_cksum {
     return sprintf("%lu",${crc});;
 }
 
+# users_table(&users)
+sub users_table
+{
+local (@ginfo, %gidgrp);
+&my_setgrent();
+while(@ginfo = &my_getgrent()) {
+	$gidgrp{$ginfo[2]} = $ginfo[0];
+	}
+&my_endgrent();
+print "<form action=mass_delete_user.cgi method=post>\n";
+print "<table width=100% border>\n";
+print "<tr $tb> ",
+      "<td><br></td> ",
+      "<td><b>$text{'user'}</b></td> ",
+      "<td><b>$text{'uid'}</b></td> ",
+      "<td><b>$text{'gid'}</b></td> ",
+      "<td><b>$text{'real'}</b></td> ",
+      "<td><b>$text{'home'}</b></td> ",
+      "<td><b>$text{'shell'}</b></td> </tr>\n";
+local $u;
+foreach $u (@{$_[0]}) {
+	$u->{'real'} =~ s/,.*$// if ($config{'extra_real'});
+	print "<tr $cb> ",
+	      "<td><input type=checkbox name=d ",
+	      "value='$u->{'user'}'></td> ",
+	      "<td><a href='edit_user.cgi?",
+	      "num=$u->{'num'}'>",&html_escape($u->{'user'}),
+	      "</a></td> <td>$u->{'uid'}</td> ",
+	      "<td>",$gidgrp{$u->{'gid'}}||$u->{'gid'},"</td> ",
+	      "<td>",&ifblank($u->{'real'}),"</td> ",
+	      "<td>",&ifblank($u->{'home'}),"</td> ",
+	      "<td>",&ifblank($u->{'shell'}),"</td> </tr>\n";
+	}
+print "</table>\n";
+print "<input type=submit value='$text{'index_mass'}'>\n";
+print "</form>\n";
+}
+
+# groups_table(&groups)
+sub groups_table
+{
+print "<form action=mass_delete_group.cgi method=post>\n";
+print "<table border width=100%>\n";
+print "<tr $tb> <td><br></td> ",
+      "<td><b>$text{'gedit_group'}</b></td> ",
+      "<td><b>$text{'gedit_gid'}</b></td> ",
+      "<td><b>$text{'gedit_members'}</b></td> </tr>\n";
+local $g;
+foreach $g (@{$_[0]}) {
+	local $members = join(" ", split(/,/, $g->{'members'}));
+	print "<tr $cb>\n";
+	print "<td><input type=checkbox name=d value='$g->{'group'}'></td>\n";
+	print "<td><a href='edit_group.cgi?",
+	      "num=$g->{'num'}'>",&html_escape($g->{'group'}),
+	      "</a></td> <td>$g->{'gid'}</td> ",
+	      "<td>",&ifblank($members),"</td> </tr>\n";
+	}
+print "</table>\n";
+print "<input type=submit value='$text{'index_gmass'}'>\n";
+print "</form>\n";
+}
+
+sub ifblank
+{
+return $_[0] ? &html_escape($_[0]) : "&nbsp;";
+}
+
+# date_input(day, month, year, prefix)
+sub date_input
+{
+print "<input name=$_[3]d size=3 value='$_[0]'>";
+print "/<select name=$_[3]m>\n";
+local $m;
+foreach $m (1..12) {
+	printf "<option value=%d %s>%s\n",
+		$m, $_[1] eq $m ? 'selected' : '', $text{"smonth_$m"};
+	}
+print "</select>";
+print "/<input name=$_[3]y size=5 value='$_[2]'>";
+print &date_chooser_button("$_[3]d", "$_[3]m", "$_[3]y");
+}
+
 1;
