@@ -5,31 +5,12 @@
 $no_acl_check++;
 require './webmin-lib.pl';
 
-# Get the update source
-if ($config{'upsource'}) {
-	($host, $port, $page, $ssl) = &parse_http_url($config{'upsource'});
-	$host || die "Invalid update source URL!";
-	}
-else {
-	$host = $update_host;
-	$port = $update_port;
-	$page = $update_page;
-	}
-
-# Retrieve the updates list (format is  module version url support description )
-$temp = &tempname();
-&http_download($host, $port, $page, $temp, undef, undef, $ssl);
-open(UPDATES, $temp);
-while(<UPDATES>) {
-	if (/^([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+(.*)/) {
-		push(@updates, [ $1, $2, $3, $4, $5 ]);
-		}
-	}
-close(UPDATES);
-unlink($temp);
+# Fetch the updates
+($updates, $host, $port, $page, $ssl) =
+	&fetch_updates($config{'upsource'} || $update_url);
 
 # Go through the results
-foreach $u (@updates) {
+foreach $u (@$updates) {
 	# Skip modules that are not for this version of Webmin, IF the module
 	# is a core module or is not installed
 	local %minfo = &get_module_info($u->[0]);
