@@ -3,6 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
 
 @cs_codes = ( 'cs_page', 'cs_text', 'cs_table', 'cs_header', 'cs_link' );
 @cs_names = map { $text{$_} } @cs_codes;
@@ -10,6 +11,7 @@ do '../web-lib.pl';
 $update_host = "www.webmin.com";
 $update_port = 80;
 $update_page = "/updates/updates.txt";
+$update_url = "http://$update_host:$update_port$update_page";
 
 $webmin_key_email = "jcameron\@webmin.com";
 $webmin_key_fingerprint = "1719 003A CE3E 5A41 E2DE  70DF D97A 3AE9 11F6 3C51";
@@ -28,7 +30,8 @@ $default_key_size = "512";
 
 sub setup_ca
 {
-local $conf = `cat $root_directory/acl/openssl.cnf`;
+local $adir = &module_root_directory("acl");
+local $conf = `cat $adir/openssl.cnf`;
 local $acl = "$config_directory/acl";
 $conf =~ s/DIRECTORY/$acl/g;
 &lock_file("$acl/openssl.cnf");
@@ -87,6 +90,7 @@ sub install_webmin_module
 local ($file, $need_unlink, $nodeps, $grant) = @_;
 local (@mdescs, @mdirs, @msizes);
 local (@newmods, $m);
+local $install_root_directory = $gconfig{'install_root'} || $root_directory;
 
 # Uncompress the module file if needed
 open(MFILE, $file);
@@ -162,7 +166,7 @@ if ($type eq 'rpm' && $file =~ /\.rpm$/i &&
 		return &text('install_eirpm', "<tt>$out</tt>");
 		}
 
-	$mdirs[0] = "$root_directory/$name";
+	$mdirs[0] = &module_root_directory($name);
 	if (%minfo = &get_module_info($name)) {
 		# Get the new module info
 		$mdescs[0] = $minfo{'desc'};
@@ -259,7 +263,7 @@ else {
 							"<tt>$dver</tt>");
 						}
 					}
-				elsif (!-r "$root_directory/$dep/module.info" &&
+				elsif (!&foreign_exists($dep) &&
 				       !$mods{$dep}) {
 					# Depends on some other module
 					$err = &text('install_edep',
@@ -286,24 +290,23 @@ else {
 	# Delete modules or themes being replaced
 	local @grantmods;
 	foreach $m (@realmods) {
-		push(@grantmods, $m) if (!-d "$root_directory/$m");
-		system("rm -rf '$root_directory/$m' 2>&1 >/dev/null") if ($m ne 'webmin');
+		push(@grantmods, $m) if (!&foreign_exists($m));
+		if ($m ne "webmin") {
+			system("rm -rf ".quotemeta("$install_root_directory/$m")." 2>&1 >/dev/null");
+			}
 		}
 
 	# Extract all the modules and update perl path and ownership
-	local $out = `cd $root_directory ; tar xf "$file" 2>&1 >/dev/null`;
+	local $out = `cd $install_root_directory ; tar xf "$file" 2>&1 >/dev/null`;
 	if ($?) {
 		unlink($file) if ($need_unlink);
 		return &text('install_eextract', $out);
 		}
 	if ($need_unlink) { unlink($file); }
-	local $perl;
-	open(PERL, "$module_root_directory/index.cgi");
-	<PERL> =~ /^#!(\S+)/; $perl = $1;
-	close(PERL);
+	local $perl = &get_perl_path();
 	local @st = stat("$module_root_directory/index.cgi");
 	foreach $moddir (keys %mods) {
-		local $pwd = "$root_directory/$moddir";
+		local $pwd = &module_root_directory($moddir);
 		if ($hasfile{$moddir,"module.info"}) {
 			local %minfo = &get_module_info($moddir);
 			push(@mdescs, $minfo{'desc'});
@@ -321,8 +324,8 @@ else {
 			&webmin_log("tinstall", undef, $moddir,
 				    { 'desc' => $tinfo{'desc'} });
 			}
-		system("cd $root_directory ; (find $pwd -name '*.cgi' ; find $pwd -name '*.pl') 2>/dev/null | $perl $root_directory/perlpath.pl $perl -");
-		system("cd $root_directory ; chown -R $st[4]:$st[5] $pwd");
+		system("cd $install_root_directory ; (find $pwd -name '*.cgi' ; find $pwd -name '*.pl') 2>/dev/null | $perl $root_directory/perlpath.pl $perl -");
+		system("cd $install_root_directory ; chown -R $st[4]:$st[5] $pwd");
 		}
 
 	# Copy appropriate config file from modules to /etc/webmin
@@ -335,7 +338,7 @@ else {
 
 # Run post-install scripts
 foreach $m (@newmods) {
-	next if (!-r "$root_directory/$m/postinstall.pl");
+	next if (!-r &module_root_directory($m)."/postinstall.pl");
 	eval {
 		&foreign_require($m, "postinstall.pl");
 		&foreign_call($m, "module_install");
@@ -396,7 +399,7 @@ if ($minfo{'clone'}) {
 	# Deleting a clone
 	local %cinfo;
 	&read_file("$config_directory/$m/clone", \%cinfo);
-	unlink("$root_directory/$m");
+	unlink(&module_root_directory($m));
 	system("rm -rf $config_directory/$m");
 	if ($gconfig{'theme'}) {
 		unlink("$root_directory/$gconfig{'theme'}/$m");
@@ -406,25 +409,29 @@ if ($minfo{'clone'}) {
 else {
 	# Delete any clones of this module
 	local @clones;
-	local @mst = stat("$root_directory/$m");
-	opendir(DIR, $root_directory);
-	foreach $l (readdir(DIR)) {
-		@lst = stat("$root_directory/$l");
-		if (-l "$root_directory/$l" && $lst[1] == $mst[1]) {
-			unlink("$root_directory/$l");
-			system("rm -rf $config_directory/$l");
-			push(@clones, $l);
+	local $mdir = &module_root_directory($m);
+	local @mst = stat($mdir);
+	local $r;
+	foreach $r (@root_directories) {
+		opendir(DIR, $r);
+		foreach $l (readdir(DIR)) {
+			@lst = stat("$r/$l");
+			if (-l "$r/$l" && $lst[1] == $mst[1]) {
+				unlink("$r/$l");
+				system("rm -rf $config_directory/$l");
+				push(@clones, $l);
+				}
 			}
+		closedir(DIR);
 		}
-	closedir(DIR);
 
-	open(TYPE, "$root_directory/$m/install-type");
+	open(TYPE, "$mdir/install-type");
 	chop($type = <TYPE>);
 	close(TYPE);
 
 	# Run the module's uninstall script
 	if (&check_os_support(\%minfo) &&
-	    -r "$root_directory/$m/uninstall.pl") {
+	    -r "$mdir/uninstall.pl") {
 		eval {
 			&foreign_require($m, "uninstall.pl");
 			&foreign_call($m, "module_uninstall");
@@ -432,17 +439,16 @@ else {
 		}
 
 	# Deleting the real module
-	$pwd = "$root_directory/$m";
-	local $size = &disk_usage_kb($pwd);
+	local $size = &disk_usage_kb($mdir);
 	$mdesc = &text('delete_desc2', "<b>$minfo{'desc'}</b>",
-			   "<tt>$pwd</tt>", $size);
+			   "<tt>$mdir</tt>", $size);
 	if ($type eq 'rpm') {
 		# This module was installed from an RPM .. rpm -e it
 		&system_logged("rpm -e wbm-$m");
 		}
 	else {
 		# Module was installed from a .wbm file .. just rm it
-		&system_logged("rm -rf $root_directory/$m");
+		&system_logged("rm -rf ".quotemeta($mdir));
 		}
 
 	if ($_[1]) {
@@ -589,7 +595,7 @@ return \@rv;
 sub third_chooser_button
 {
 local $form = @_ > 1 ? $_[1] : 0;
-return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"third_chooser.cgi?mod=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=700,height=300\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
+return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"third_chooser.cgi?mod=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=700,height=300\"); chooser.ifield = ifield; window.ifield = ifield' value=\"$text{'mods_thsel'}\">\n";
 }
 
 # get_webmin_base_version()
@@ -600,10 +606,10 @@ return &base_version(&get_webmin_version());
 }
 
 # base_version()
-# Rounds a version number to the nearest .01
+# Rounds a version number down to the nearest .01
 sub base_version
 {
-return sprintf("%.2f0", $_[0]);
+return sprintf("%.2f0", $_[0] - 0.005);
 }
 
 $newmodule_users_file = "$config_directory/newmodules";
@@ -667,4 +673,28 @@ foreach $s (split(/\s+/, $_[0]->{'sockets'})) {
 return @sockets;
 }
 
+# fetch_updates(url)
+# Returns a list of updates from some URL, or calls &error.
+# Format is  module version url support description
+sub fetch_updates
+{
+local ($host, $port, $page, $ssl) = &parse_http_url($_[0]);
+$host || &error($text{'update_eurl'});
+
+local $temp = &tempname();
+local @updates;
+&http_download($host, $port, $page, $temp, undef, undef, $ssl);
+open(UPDATES, $temp);
+while(<UPDATES>) {
+	if (/^([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+([^\t]+)\t+(.*)/) {
+		push(@updates, [ $1, $2, $3, $4, $5 ]);
+		}
+	}
+close(UPDATES);
+unlink($temp);
+@updates || &error($text{'update_efile'});
+
+return ( \@updates, $host, $port, $page, $ssl );
+}
+
 1;
