@@ -2,6 +2,10 @@
 
 do '../web-lib.pl';
 &init_config();
+require '../ui-lib.pl';
+%access = &get_module_acl();
+%access_mods = map { $_, 1 } split(/\s+/, $access{'mods'});
+%access_users = map { $_, 1 } split(/\s+/, $access{'users'});
 
 sub parse_logline
 {
@@ -27,6 +31,10 @@ if ($_[0] =~ /^(\d+)\.(\S+)\s+\[.*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+"
 		$param{$k} =~ s/%(..)/pack("c",hex($1))/ge;
 		}
 	$rv->{'param'} = \%param;
+	if ($rv->{'script'} =~ /^(\S+):(\S+)$/) {
+		$rv->{'script'} = $2;
+		$rv->{'webmin'} = $1;
+		}
 	return $rv;
 	}
 else {
@@ -58,5 +66,17 @@ while(open(DIFF, "$ENV{'WEBMIN_VAR'}/diffs/$act->{'id'}.$i")) {
 return @rv;
 }
 
+# can_user(username)
+sub can_user
+{
+return $access_users{'*'} || $access_users{$_[0]};
+}
+
+# can_mod(module)
+sub can_mod
+{
+return $access_mods{'*'} || $access_mods{$_[0]};
+}
+
 1;
 
